-- ======================================================
--   Complete functional simulation for Example 3
-- ======================================================
--
--  Module      : TB_EXAMPLE3
--  Date        : 28/02/2002
--  Author      : R. Williams - HUNT ENGINEERING
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    28-02-02    First Written
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    14-10-02    Modified for use with 6 FIFO
--                                     interface components.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;

library sim_lib;
  use sim_lib.all;


entity TESTBENCH is
end TESTBENCH;

architecture BENCH of TESTBENCH is

  component TOP
    port (
      -- PADS - HE_WCLK
      doclk        : in  std_logic;
      f0           : out std_logic;
      -- PADS - HE_WR_6F
      dof_ff       : in  std_logic_vector(5 downto 0);
      dof_af       : in  std_logic_vector(5 downto 0);
      dof_wen      : out std_logic_vector(5 downto 0);
      do           : out std_logic_vector(31 downto 0);
      -- PADS - HE_RCLK
      diclk        : in  std_logic;
      f1           : out std_logic;
      -- PADS - HE_RD_6F
      dif_ef       : in  std_logic_vector(5 downto 0);
      dif_ae       : in  std_logic_vector(5 downto 0);
      dif_ren      : out std_logic_vector(5 downto 0);
      dif_oen      : out std_logic_vector(5 downto 0);
      di           : in  std_logic_vector(31 downto 0);
      -- PADS - HSB
      vpd          : inout std_logic_vector(7 downto 0);
      vinit        : inout std_logic;
      vcs          : inout std_logic;
      vwrite       : inout std_logic;
      vbusy        : inout std_logic;
      -- PADS - HE_SCLKA
      enc_a_p      : out std_logic;
      enc_a_n      : out std_logic;
      encode_a_p   : in  std_logic;
      encode_a_n   : in  std_logic;
      -- PADS - HE_SCLKB
      enc_b_p      : out std_logic;
      enc_b_n      : out std_logic;
      encode_b_p   : in  std_logic;
      encode_b_n   : in  std_logic;
      -- PADS - HE_ADA
      ada          : in  std_logic_vector(11 downto 0);
      otra         : in  std_logic;
      -- PADS - HE_ADB
      adb          : in  std_logic_vector(11 downto 0);
      otrb         : in  std_logic;
      -- PADS - HE_DAA
      wrt0         : out std_logic;
      trig0        : out std_logic;
      daa          : out std_logic_vector(13 downto 0);
      -- PADS - HE_DAB
      wrt1         : out std_logic;
      trig1        : out std_logic;
      dab          : out std_logic_vector(13 downto 0);
      -- CONTROL signals
      config       : in  std_logic;
      reset_pin    : in  std_logic;
      udpres       : out std_logic;
      -- OSC/CLOCKS
      osc0_pin     : in  std_logic;
      osc1_pin     : in  std_logic;
      osc2_pin     : in  std_logic;
      osc3_pin     : in  std_logic;
      clkin0       : in  std_logic;
      clkin1       : in  std_logic;
      clki2        : in  std_logic;
      clki3        : in  std_logic;
      clkout       : out std_logic;
      pclk_p       : in  std_logic;
      pclk_n       : in  std_logic;
      -- MISC I/O
      led          : out std_logic_vector(4 downto 0);
      digital_io   : inout std_logic_vector(7 downto 0);
      cid          : in std_logic_vector(3 downto 0);
      mid          : in std_logic_vector(3 downto 0);
      umi          : inout std_logic_vector(3 downto 0);
      addr_flagsel : in  std_logic;
      booten       : out std_logic;
      -- SERIAL I/O
      r1out        : in  std_logic;
      r2out        : in  std_logic;
      t1in         : out std_logic;
      t2in         : out std_logic;
      rs485_232    : out std_logic;
      hdplx        : out std_logic;
      fast         : out std_logic;
      qttl         : in  std_logic;
      dttl         : out std_logic
  );
  end component;

  component SIM_RD_6F
    generic (
      FILE_IN0 : string;
      FILE_IN1 : string;
      FILE_IN2 : string;
      FILE_IN3 : string;
      FILE_IN4 : string;
      FILE_IN5 : string
    );
    port (
      GO      : in  boolean;                       -- start module activity
      CLK     : in  std_logic;                     -- FIFO clock
      DIF_REN : in  std_logic_vector(5 downto 0);  -- read enables
      DIF_OEN : in  std_logic_vector(5 downto 0);  -- output enable
      ENDF0   : out boolean;                       -- end of file 0
      ENDF1   : out boolean;                       -- end of file 1
      ENDF2   : out boolean;                       -- end of file 2
      ENDF3   : out boolean;                       -- end of file 3
      ENDF4   : out boolean;                       -- end of file 4
      ENDF5   : out boolean;                       -- end of file 5
      DIF_EF  : out std_logic_vector(5 downto 0);  -- empty flags
      DIF_AE  : out std_logic_vector(5 downto 0);  -- almost empty flags
      DI      : out std_logic_vector(31 downto 0)  -- read data
    );
  end component;

  component SIM_WR_6F
    generic (
      FILE_OUT0 : string;
      FILE_OUT1 : string;
      FILE_OUT2 : string;
      FILE_OUT3 : string;
      FILE_OUT4 : string;
      FILE_OUT5 : string
    );
    port (
      CLK     : in  std_logic;                      -- FIFO clock
      DOF_WEN : in  std_logic_vector(5 downto 0);   -- write enables
      DO      : in  std_logic_vector(31 downto 0);  -- write data
      DOF_FF  : out std_logic_vector(5 downto 0);   -- full flags
      DOF_AF  : out std_logic_vector(5 downto 0)    -- almost full flags
    );
  end component;

  component SIM_MSG
    port (
      VPD    : inout std_logic_vector(7 downto 0);  -- data
      VINIT  : inout std_logic;                     -- address strobe
      VCS    : inout std_logic;                     -- data strobe
      VWRITE : inout std_logic;                     -- read/nwrite
      VBUSY  : inout std_logic;                     -- ready/nbusy
      --
      START  : in  boolean;                         -- transaction -> start operation
      OP     : in  std_logic;                       -- 1/read - 0/write
      ADDR   : in  std_logic_vector(7 downto 0);    -- address
      DIN    : in  std_logic_vector(7 downto 0);    -- write data
      DOUT   : out std_logic_vector(7 downto 0);    -- read data
      READY  : out boolean                          -- 0/busy - 1/finished
    );
  end component;

  -- TOP I/Os
  signal DOCLK        : std_logic;
  signal F0           : std_logic;
  signal DOF_FF       : std_logic_vector(5 downto 0);
  signal DOF_AF       : std_logic_vector(5 downto 0);
  signal DOF_WEN      : std_logic_vector(5 downto 0);
  signal DO           : std_logic_vector(31 downto 0);
  signal DICLK        : std_logic;
  signal F1           : std_logic;
  signal DIF_EF       : std_logic_vector(5 downto 0);
  signal DIF_AE       : std_logic_vector(5 downto 0);
  signal DIF_REN      : std_logic_vector(5 downto 0);
  signal DIF_OEN      : std_logic_vector(5 downto 0);
  signal DI           : std_logic_vector(31 downto 0);
  signal VPD          : std_logic_vector(7 downto 0);
  signal VINIT        : std_logic;
  signal VCS          : std_logic;
  signal VWRITE       : std_logic;
  signal VBUSY        : std_logic;
  signal ENC_A_P      : std_logic := '0';
  signal ENC_A_N      : std_logic;
  signal ENCODE_A_P   : std_logic;
  signal ENCODE_A_N   : std_logic;
  signal ENC_B_P      : std_logic := '0';
  signal ENC_B_N      : std_logic;
  signal ENCODE_B_P   : std_logic;
  signal ENCODE_B_N   : std_logic;
  signal ADA          : std_logic_vector(11 downto 0);
  signal OTRA         : std_logic;
  signal ADB          : std_logic_vector(11 downto 0);
  signal OTRB         : std_logic;
  signal WRT0         : std_logic;
  signal TRIG0        : std_logic;
  signal DAA          : std_logic_vector(13 downto 0);
  signal WRT1         : std_logic;
  signal TRIG1        : std_logic;
  signal DAB          : std_logic_vector(13 downto 0);
  signal CONFIG       : std_logic;
  signal RESET_PIN    : std_logic;
  signal UDPRES       : std_logic;
  signal OSC0_PIN     : std_logic;
  signal OSC1_PIN     : std_logic;
  signal OSC2_PIN     : std_logic;
  signal OSC3_PIN     : std_logic := '0';
  signal CLKIN0       : std_logic;
  signal CLKIN1       : std_logic;
  signal CLKI2        : std_logic;
  signal CLKI3        : std_logic := '0';
  signal CLKOUT       : std_logic;
  signal PCLK_P       : std_logic;
  signal PCLK_N       : std_logic;
  signal LED          : std_logic_vector(4 downto 0);
  signal DIGITAL_IO   : std_logic_vector(7 downto 0);
  signal CID          : std_logic_vector(3 downto 0);
  signal MID          : std_logic_vector(3 downto 0);
  signal UMI          : std_logic_vector(3 downto 0);
  signal ADDR_FLAGSEL : std_logic;
  signal BOOTEN       : std_logic;
  signal R1OUT        : std_logic;
  signal R2OUT        : std_logic;
  signal T1IN         : std_logic;
  signal T2IN         : std_logic;
  signal RS485_232    : std_logic;
  signal HDPLX        : std_logic;
  signal FAST         : std_logic;
  signal QTTL         : std_logic;
  signal DTTL         : std_logic;

  -- oscillator period
  constant period  : time := 10 ns;
  -- ADC sample clock period
  constant ADC_period : time := 10 ns;
  -- DAC sample clock period
  constant DAC_period : time := 10 ns;
  -- input FIFO ID
  constant FIFO_IN  : std_logic_vector(5 downto 0) := "000001";
  -- output FIFO ID
  constant FIFO_OUT : std_logic_vector(5 downto 0) := "000010";

  signal GO        : boolean;
  signal ENDF0     : boolean;
  signal ENDF1     : boolean;
  signal ENDF2     : boolean;
  signal ENDF3     : boolean;
  signal ENDF4     : boolean;
  signal ENDF5     : boolean;
  signal MSG_START : boolean;
  signal MSG_OP    : std_logic := '1';
  signal MSG_ADDR  : std_logic_vector(7 downto 0);
  signal MSG_DIN   : std_logic_vector(7 downto 0);
  signal MSG_DOUT  : std_logic_vector(7 downto 0);
  signal MSG_READY : boolean;

  signal CHANNEL_A : std_logic_vector(13 downto 0);
  signal CHANNEL_B : std_logic_vector(13 downto 0);

begin

  OSC3_PIN <= NOT OSC3_PIN after (period/2);

  -- ADC A Sample Clock
  ENC_A_P <= NOT ENC_A_P after (ADC_period/2);

  -- ADC B Sample Clock
  ENC_B_P <= NOT ENC_B_P after (ADC_period/2);

  -- DAC Sample Clock
  CLKI3 <= NOT CLKI3 after (DAC_period/2);

-- ----------------------------------------
--  FPGA
-- ----------------------------------------

  UUT: TOP
    port map (
      -- PADS - HE_WCLK
      doclk        => DOCLK,
      f0           => F0,
      -- PADS - HE_WR_6F
      dof_ff       => DOF_FF,
      dof_af       => DOF_AF,
      dof_wen      => DOF_WEN,
      do           => DO,
      -- PADS - HE_RCLK
      diclk        => DICLK,
      f1           => F1,
      -- PADS - HE_RD_6F
      dif_ef       => DIF_EF,
      dif_ae       => DIF_AE,
      dif_ren      => DIF_REN,
      dif_oen      => DIF_OEN,
      di           => DI,
      -- PADS - HSB
      vpd          => VPD,
      vinit        => VINIT,
      vcs          => VCS,
      vwrite       => VWRITE,
      vbusy        => VBUSY,
      -- PADS - HE_SCLKA
      enc_a_p      => ENC_A_P,
      enc_a_n      => ENC_A_N,
      encode_a_p   => ENCODE_A_P,
      encode_a_n   => ENCODE_A_N,
      -- PADS - HE_SCLKB
      enc_b_p      => ENC_B_P,
      enc_b_n      => ENC_B_N,
      encode_b_p   => ENCODE_B_P,
      encode_b_n   => ENCODE_B_N,
      -- PADS - HE_ADA
      ada          => ADA,
      otra         => OTRA,
      -- PADS - HE_ADB
      adb          => ADB,
      otrb         => OTRB,
      -- PADS - HE_DAA
      wrt0         => WRT0,
      trig0        => TRIG0,
      daa          => DAA,
      -- PADS - HE_DAB
      wrt1         => WRT1,
      trig1        => TRIG1,
      dab          => DAB,
      -- CONTROL signals
      config       => CONFIG,
      reset_pin    => RESET_PIN,
      udpres       => UDPRES,
      -- OSC/CLOCKS
      osc0_pin     => OSC0_PIN,
      osc1_pin     => OSC1_PIN,
      osc2_pin     => OSC2_PIN,
      osc3_pin     => OSC3_PIN,
      clkin0       => CLKIN0,
      clkin1       => CLKIN1,
      clki2        => CLKI2,
      clki3        => CLKI3,
      clkout       => CLKOUT,
      pclk_p       => PCLK_P,
      pclk_n       => PCLK_N,
      -- MISC I/O
      led          => LED,
      digital_io   => DIGITAL_IO,
      cid          => CID,
      mid          => MID,
      umi          => UMI,
      addr_flagsel => ADDR_FLAGSEL,
      booten       => BOOTEN,
      -- SERIAL I/O
      r1out        => R1OUT,
      r2out        => R2OUT,
      t1in         => T1IN,
      t2in         => T2IN,
      rs485_232    => RS485_232,
      hdplx        => HDPLX,
      fast         => FAST,
      qttl         => QTTL,
      dttl         => DTTL );

-- ----------------------------------------
--  OUTPUT FIFO
-- ----------------------------------------
-- TOP outputs : F0, DOF_WEN, DO
-- TOP inputs  : DOCLK, DOF_FF, DOF_AF

  DOCLK <= F0;

  iWR6F : SIM_WR_6F
    generic map(
      FILE_OUT0 => "..\Sim\Fifo_Adc0.TXT",
      FILE_OUT1 => "..\Sim\Fifo_Adc1.TXT",
      FILE_OUT2 => "..\Sim\Fifo_Adc2.TXT",
      FILE_OUT3 => "..\Sim\Fifo_Adc3.TXT",
      FILE_OUT4 => "..\Sim\Fifo_Adc4.TXT",
      FILE_OUT5 => "..\Sim\Fifo_Adc5.TXT" )
    port map (
      CLK     => F0,
      DOF_WEN => DOF_WEN,
      DO      => DO,
      DOF_FF  => DOF_FF,
      DOF_AF  => DOF_AF );

-- ----------------------------------------
--  INPUT FIFO
-- ----------------------------------------
-- TOP outputs : F1, DIF_REN, DIF_OEN
-- TOP inputs  : DICLK, DIF_EF, DIF_AE, DI

  DICLK <= F1;

  iRD6F : SIM_RD_6F
    generic map (
      FILE_IN0 => "..\Sim\Fifo_Dac0.TXT",
      FILE_IN1 => "..\Sim\Fifo_Dac1.TXT",
      FILE_IN2 => "..\Sim\Fifo_Dac2.TXT",
      FILE_IN3 => "..\Sim\Fifo_Dac3.TXT",
      FILE_IN4 => "..\Sim\Fifo_Dac4.TXT",
      FILE_IN5 => "..\Sim\Fifo_Dac5.TXT" )
    port map (
      GO      => GO,
      CLK     => F1,
      DIF_REN => DIF_REN,
      DIF_OEN => DIF_OEN,
      ENDF0   => ENDF0,
      ENDF1   => ENDF1,
      ENDF2   => ENDF2,
      ENDF3   => ENDF3,
      ENDF4   => ENDF4,
      ENDF5   => ENDF5,
      DIF_EF  => DIF_EF,
      DIF_AE  => DIF_AE,
      DI      => DI );

-- ----------------------------------------
--  HSB MESSAGE INTERFACE
-- ----------------------------------------
-- TOP inouts : VPD, VINIT, VCS, VWRITE, VBUSY

  iMSG : SIM_MSG
    port map (
      VPD    => VPD,
      VINIT  => VINIT,
      VCS    => VCS,
      VWRITE => VWRITE,
      VBUSY  => VBUSY,
      --
      START  => MSG_START,
      OP     => MSG_OP,
      ADDR   => MSG_ADDR,
      DIN    => MSG_DIN,
      DOUT   => MSG_DOUT,
      READY  => MSG_READY );

-- ----------------------------------------
--  Channel A - ADC
-- ----------------------------------------
-- TOP outputs : ENC_A_P, ENC_A_N
-- TOP inputs  : ENCODE_A_P, ENCODE_A_N, ADA, OTRA

  ENCODE_A_P <= ENC_A_P;
  ENCODE_A_N <= ENC_A_N;

  process(ENC_A_P)
  begin
    if rising_edge(ENC_A_P) then
      ADA <= CHANNEL_A(11 downto 0) after 1 ns;
    end if;
  end process;

  OTRA <= '0';

-- ----------------------------------------
--  Channel B - ADC
-- ----------------------------------------
-- TOP outputs : ENC_B_P, ENC_B_N
-- TOP inputs  : ENCODE_B_P, ENCODE_B_N, ADB, OTRB

  ENCODE_B_P <= ENC_B_P;
  ENCODE_B_N <= ENC_B_N;

  process(ENC_B_P)
  begin
    if rising_edge(ENC_B_P) then
      ADB <= CHANNEL_B(11 downto 0) after 1 ns;
    end if;
  end process;

  OTRB <= '0';

-- ----------------------------------------
--  Channel A - DAC
-- ----------------------------------------
-- TOP outputs : WRT0, TRIG0, DAA
  process(TRIG0)
  begin
    if rising_edge(TRIG0) then
      CHANNEL_A <= DAA after 1 ns;
    end if;
  end process;

-- ----------------------------------------
--  Channel B - DAC
-- ----------------------------------------
-- TOP outputs : WRT1, TRIG1, DAB
  process(TRIG1)
  begin
    if rising_edge(TRIG1) then
      CHANNEL_B <= DAB after 1 ns;
    end if;
  end process;

-- ----------------------------------------
--  STIMULI
-- ----------------------------------------

  process
  begin
    GO <= false;

    RESET_PIN <= '0';
    wait for (period*5);
    RESET_PIN <= '1';
    wait for (period*20);

    -- configure input FIFO number
    MSG_OP   <= '0';
    MSG_ADDR <= X"00";
    MSG_DIN  <= "00" & FIFO_IN;
    MSG_START <= true;
    wait until MSG_READY;
    -- configure output FIFO number
    MSG_OP   <= '0';
    MSG_ADDR <= X"01";
    MSG_DIN  <= "00" & FIFO_OUT;
    MSG_START <= true;
    wait until MSG_READY;
    -- select 2 DAC channels
    MSG_OP   <= '0';
    MSG_ADDR <= X"03";
    MSG_DIN  <= "00000001";
    MSG_START <= true;
    wait until MSG_READY;

    -- read back input FIFO number
    MSG_OP   <= '1';
    MSG_ADDR <= X"00";
    MSG_START <= true;
    wait until MSG_READY;
    -- read back output FIFO number
    MSG_OP   <= '1';
    MSG_ADDR <= X"01";
    MSG_START <= true;
    wait until MSG_READY;

    -- SW reset
    wait for (period*20);
    RESET_PIN <= '0';
    wait for (period*5);
    RESET_PIN <= '1';
    wait for (period*20);

    -- launch FIFO read
    GO <= true;

    -- end of download
    wait until ENDF0;
    wait until ENDF1;
    wait until ENDF2;
    wait until ENDF3;
    wait until ENDF4;
    wait until ENDF5;

    wait for (period*2500);

    assert false report "End of simulation" severity failure;
  end process;

end BENCH;

