-- ==============================================================
-- FILE : ADC_DATA.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of Example 3, for the HERON-IO2V2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : ADC_DATA
--  Date        : 27/02/2002
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : ADC data from 1 or 2 channels
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    27-02-02    First Written
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity ADC_DATA is
    Port ( ADC_A     : in std_logic_vector(11 downto 0);
           ADC_B     : in std_logic_vector(11 downto 0);
           OTR_A     : in std_logic;
           OTR_B     : in std_logic;
           A2D_2CH   : in std_logic;
           SCLK      : in std_logic;
           RST       : in std_logic;
           OVERFLOW  : in std_logic;
           DVALID    : out std_logic;
           ADC_FIFO  : out std_logic_vector(31 downto 0)
           );
end ADC_DATA;

architecture RTL of ADC_DATA is

signal DVALID_A : std_logic;
signal DVALID_B : std_logic;

begin

  DVALID <= DVALID_A;

  process(RST, SCLK)
  begin
    if RST='1' then
      ADC_FIFO <= (others => '0');
      DVALID_A <= '0';
      DVALID_B <= '0';
    elsif rising_edge(SCLK) then
      DVALID_B <= DVALID_A;
      DVALID_A <= '0';
      if (A2D_2CH='1' or DVALID_B='0') then
        ADC_FIFO(15 downto 0) <= '0' & OVERFLOW & (ADC_A(11) xor ADC_A(10))
                                     & OTR_A & ADC_A(11 downto 0);
      end if;
      if (A2D_2CH='1' or DVALID_A='0') then
        if A2D_2CH='0' then
          ADC_FIFO(31 downto 16) <= '0' & OVERFLOW & (ADC_A(11) xor ADC_A(10))
                                        & OTR_A & ADC_A(11 downto 0);
        else
          ADC_FIFO(31 downto 16) <= '0' & OVERFLOW & (ADC_B(11) xor ADC_B(10))
                                        & OTR_B & ADC_B(11 downto 0);
        end if;
        DVALID_A <= '1';
      end if;
    end if;
  end process;

end RTL;

