-- ==============================================================
-- FILE : LED.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of Example 3, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : LED
--  Date        : 27/02/2002
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : LED Control
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    27-02-02    First Written
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity LED_CTRL is
  port (
    FCLK : in  std_logic;
    SCLK : in  std_logic;
    FF   : in  std_logic;
    LED  : out std_logic
  );
end LED_CTRL;


architecture RTL of LED_CTRL is

  signal LEDcount : unsigned(23 downto 0);

  signal FF_REG : std_logic;
  signal CLEAR  : std_logic;
  signal COUNT  : std_logic;

begin

  process(SCLK)
  begin
    if rising_edge(SCLK) then
      FF_REG <= FF OR (FF_REG AND (NOT CLEAR));
    end if;
  end process;

  process(FCLK)
  begin
    if rising_edge(FCLK) then
      CLEAR <= FF_REG;
    end if;
  end process;

  process(FCLK)
  begin
    if rising_edge(FCLK) then
      COUNT <= (NOT LEDcount(23));
    end if;
  end process;

  process(CLEAR, FCLK)
  begin
    if CLEAR='1' then
      LEDcount <= (others=>'0');
    elsif rising_edge(FCLK) then
      if COUNT='1' then
        LEDcount <= LEDcount + 1;
      end if;
    end if;
  end process;

  LED <= LEDcount(23);

end RTL;
