/************************************************************/
/*                                                          */
/* THIS IS A DEMO PROGRAM FOR THE HUNT ENGINEERING          */
/* HERON-IO2 board when using the example3 bitstream.       */
/* It does not use the O/Ps, but reads the inputs into an   */
/* SDRAM buffer, which is writen to a file using the        */
/* Server/Loader.                                           */
/* THE PROGRAM is for a C6000 module.                       */
/*                                                          */
/*                                                          */
/*                                                          */
/*     PW    ver 1.0    3/10/01   first written based on    */
/*                                example3                  */
/*                                                          */   
/************************************************************/


/* YOU need to set this value to reflect how your HERON-IO2 is connected
to your HERON DSP module. */
#define INFIFONO  2       /* FIFO through which IO2 Data is received */


#include <stdio.h>
#include <stdlib.h>
#include <std.h>
#include <math.h>


/* Use HERON-API for heron1 so you need the next include line*/
#include "heron4.h"

 
/* use unpack library for channel unpacking */
#include "HEL_Unpack.h"                            

#define NO_IP_CHANNELS 1

#define FPGA_TO_DSP 3
#define DSP_TO_FPGA 3
#define FPGA_SLOT	2  
#define USER_WRITE			8
#define USER_READ			9      
#define USER_READ_REPLY		10 
 
/* as it says number of samples per channel*/                            
#define SAMPLES_PER_IP_CHAN 1024  

#define MALLOC_SIZE 4*1024*1020  /* leave some room for the other heaps*/

/* mask for data bits */
#define IP_DATA_MASK ((unsigned short) 0xfff)
  
short inputsa[SAMPLES_PER_IP_CHAN*NO_IP_CHANNELS]; 
short inputsb[SAMPLES_PER_IP_CHAN*NO_IP_CHANNELS]; 
short sorted_ips[SAMPLES_PER_IP_CHAN*NO_IP_CHANNELS]; 
/* define some pointers that can be used to access each channel from the results*/
short *ch0_data=sorted_ips; 
#if (NO_IP_CHANNELS==2) 
short *ch1_data=sorted_ips+SAMPLES_PER_IP_CHAN;
#endif

#define BUFFER_SIZE_WORDS  SAMPLES_PER_IP_CHAN*NO_IP_CHANNELS/2 
       
/* allocate the errors array */
#pragma DATA_ALIGN(errors,4);   
short errors[2];                     

HeronFIFO *rfifo;		/* Handle for read FIFO  */
HeronHSB *hsb;          /* handle for HSB accesses */
int board,slot;


        
/********************************************************************************/
/*																				*/
/*	readback_user																*/
/*																				*/
/*	This function is used to read back from the FPGA							*/
/*																				*/
/********************************************************************************/
int readback_user(unsigned short targetslot, unsigned char *reply, int buffer_size, int user_addr)
{
	int message_type;
	unsigned char send_data[5];   
	unsigned int done;
	int status;

	send_data[0]=user_addr;
	send_data[1]=1;


   	status = HeronHsbSendMessage(hsb,USER_READ,board,targetslot,send_data,2);  
   	status = HeronHsbReceiveMessage(hsb,&message_type,board,targetslot,reply,buffer_size,&done);
	if (status == HERON_OK) 
	{
		if (message_type != USER_READ_REPLY) 
		{
			printf("SLOT%d:Error: expected user read reply   \n",slot);
			return(-1);
		}
		else 
		{       
			if (done!=1)
			{
				printf("SLOT%d:Error: expected one byte, recieved %d  \n",slot,done);
				return(-1);
			}
			else
			{	
			return(reply[0]);
			}
		}
	}

	return(status);
}    



/****************************************************/
/* makes an HSB message to send to the FPGA module	*/
/* it is a "user" type message to write to the      */
/* application FPGA. In the example1 bitstream address */
/* 0 sets the FIFO to use for input to the FPGA, and */
/* address 1 sets the FIFO to use for writes from the */
/* FPGA												*/
/************************************/


void SendUserData(unsigned short targetslot, int Address, int Data)
{
unsigned char user[2];
unsigned char rec_data[3];
int reply;
int status;

	/* Send user data */
	user[0]=Address;
	user[1]=Data;
        
	status = HeronHsbSendMessage(hsb,USER_WRITE,board,targetslot,user,2);        
           
	if(status != HERON_OK) 
		{
		printf("SLOT%d:HSB Write module failed with %x, exiting\n",targetslot, status);  
		exit(0);
		}
/* read it back to check*/        
    reply =readback_user(targetslot,rec_data,2,Address);
    if(reply!=Data)
    {
    printf("SLOT%d:error hsb readback failed with %x\n",targetslot,reply); 
    exit(0);
    }

}         
      
/****************************************************/
/* makes an HSB message to send to the FPGA module	*/
/* it is a "user" type message to write to the      */
/* application FPGA. In the example1 bitstream address */
/* 0 sets the FIFO to use for input to the FPGA, and */
/* address 1 sets the FIFO to use for writes from the */
/* FPGA												*/
/************************************/


void SendUserDatanr(unsigned short targetslot, int Address, int Data)
{
unsigned char user[2];
int status;

	/* Send user data */
	user[0]=Address;
	user[1]=Data;
        
	status = HeronHsbSendMessage(hsb,USER_WRITE,board,targetslot,user,2);        
         
	if(status != HERON_OK) 
		{
		printf("SLOT%d:HSB Write module failed with %x, exiting\n",targetslot, status);  
		exit(0);
		}

}                  
        
        
main()
{
bootloader();
}        
        


maintask (void)
{
/* WARNING the default stack size for this task ( maintask) defined in    */
/* the cdb file is 8192 bytes                                             */
/* This means that you cannot statically declare more than 2048 integers  */
/* in this task.                                                          */
/* You can increase the stack size allowed by changing the settings for   */
/* the task in the cdb file.                                              */
/* If you need more than 2048 integers, it is better to use Malloc/Free   */
/* to allocate your data buffers 					  */

int status;
int *stored;
short *shortstored; /* for writing the file*/
int *startstored; /* for testing maximum rates */
int done=0;
int temp;
int i;
FILE *fptr;
    printf("Hello from HERON-IO2 Example3a demo.\n");



slot = HeronModId() && 0xf;
board= (HeronModId() && 0xf0)>>4;            
/* open the HSB device to be able to set the FIFO number for the FPGA example1 */
hsb = HeronHsbOpen();
if (hsb == NULL) {
	printf("SLOT%d:Error: Call to HeronHsbOpen failed, error = %d\n",slot,heronerr); 
	exit(0);
}       


/* open the FIFO for reading*/
    rfifo=HeronOpenFifo(INFIFONO,"rd");
    if (rfifo==NULL)
     {
       printf("error opening FIFO %d, error is %x\n",INFIFONO,heronerr);
     }

	temp=1<<FPGA_TO_DSP;  
	SendUserData(FPGA_SLOT, 1, temp); /* set the FIFO the FPGA will use */
#if (NO_IP_CHANNELS==2)	
	SendUserDatanr(FPGA_SLOT, 2, 1); /* set 2 channels operation */
#else
	SendUserDatanr(FPGA_SLOT, 2, 0); /* set 1 channels operation */
#endif

stored= malloc(MALLOC_SIZE*sizeof(int)); 
if (stored==NULL)
	{
	printf("malloc failed\n");
	exit(0);
	}
startstored=stored; /* for the record where we started*/


    /* RELEASE THE CONFIG LINE */
    HeronConfigOff();
//while (1)  /* testing*/
//{

/* The data needs to be flowing waiting for some event*/
/* Keeping it flowing means that there is no stale data */
/* in any of the FIFOs of the system */
/* in this case we just count 50 buffers without incrementing the pointer*/


for(i=0;i<50;i++)   
    {                      
                     
	status = HeronRead(rfifo,stored,(BUFFER_SIZE_WORDS));         
	if (status != HERON_IO_IN_PROGRESS)
		{
		printf("error from read \n");
		exit(0);
		}
	status = HeronWaitIo(rfifo);
	if (status != HERON_OK)
		{
		printf("Cannot read fifo %d. Error %d,\n",INFIFONO,heronerr);
		exit(0);
		}
    }   





while(done<=(MALLOC_SIZE-BUFFER_SIZE_WORDS))    
    {                      
                     
	status = HeronRead(rfifo,stored,(BUFFER_SIZE_WORDS));         
	if (status != HERON_IO_IN_PROGRESS)
		{
		printf("error from read \n");
		exit(0);
		}
	status = HeronWaitIo(rfifo);
	if (status != HERON_OK)
		{
		printf("Cannot read fifo %d. Error %d,\n",INFIFONO,heronerr);
		exit(0);
		}
		
	done+=BUFFER_SIZE_WORDS;
	stored+=BUFFER_SIZE_WORDS;
    }   
 stored=startstored; /* set the pointer back to the beginning*/
 done=0;  /* for reading the data now*/   
 shortstored=(short *)stored;
//}
printf("captured all data\n");
fptr= fopen("c:\\io2data.txt","w");
    if(fptr==NULL)printf("error opening file\n");

printf("writing file......\n"); 
for(i=0;i<MALLOC_SIZE*2;i+=8)  /* times 2 because we are printing shorts*/   
    {                      
	fprintf(fptr,("%04x %04x %04x %04x %04x %04x %04x %04x\n"), shortstored[i], shortstored[i+1], shortstored[i+2], shortstored[i+3], shortstored[i+4], shortstored[i+5], shortstored[i+6], shortstored[i+7]); 
	/* this takes a long time because it is formatting and printing each object*/
						/* a binary file would be quicker, but harder to see.*/
	if(i%(MALLOC_SIZE*2/10)==0)printf("done 1/10th more\n");					            
    }      
    
printf("file written \n");
fclose(fptr);    
    
          
/* we cannot reach here but here's how to close the FIFOs if you did */    
	status = HeronClose(rfifo);
	if (status != HERON_OK)
	{
	printf(" Error closing fifo \n");
	exit(-1);
	}    
    
    
    
    

} /* end maintask */
