/************************************************************************/
/*																		*/
/* Prog_FIFO.c                                                          */
/*																		*/
/* Example that uses HSB to send messages to a HERON-FPGA or HERON-IO   */
/* module that is running the "Getting Started" example. These messages */
/* set up which FIFOs will be used by the FPGA.		.					*/
/*																		*/
/*	    2001 Hunt Engineering											*/
/*	    All Rights Reserved												*/
/*																		*/
/* 	Ver	Date	 	Author	Comments									*/
/*	---	----		-------	--------									*/
/*																		*/
/*	0.1	19/12/00	RASW	Created.									*/
/*  1.0 14/2/01     PW      added environment handling and command      */
/*                          line switches								*/
/*  1.1 22-2-02     PW      added support for HEART based carriers      */
/*                          like HEPC9                                  */
/*  1.2 04-8-05     JT      Support for HEBASE2 added                   */
/* 																		*/
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <math.h>
#include "heapi.h"

HE_HANDLE 		hDevice;
HE_DWORD 		Status;

HE_BYTE Slot;
HE_WORD Board=0; // The board number we will be using
int verbose;
char devname[255]; // The Host Interface Type we will be using

/* Message Type Definitions */
#define MODULE_TYPE_QUERY	1
#define MODULE_TYPE_REPLY	2
#define SEND_BITSTREAM		3
#define USER_WRITE			8
#define USER_READ			9
#define USER_READ_REPLY		10

#define PI 3.1415926535         /* So we can calculate angles in radians */  

#define INFIFONO  2       /* FIFO through which IO2 Data is received */
#define OUTFIFONO  2       /* FIFO through which IO2 Data is sent */

#define NO_IP_CHANNELS 1
#define NO_OP_CHANNELS 1


/* this example allows you to run inputs, or outputs */
/* if you select both then they will both run, but   */
/* the example is written assuming the same input and output rates*/
/* if you want to use independant input and output then using */
/* two tasks and the HeronSem functions would be better */ 
#define DOINPUT   /* comment out for no inputs */
#define DOOUTPUT  /* comment out for no outputs*/

/* as it says number of samples per channel*/                            
#define SAMPLES_PER_IP_CHAN 1024 
/* as it says number of samples per channel*/                            
#define SAMPLES_PER_OP_CHAN 1024 

short inputsa[SAMPLES_PER_IP_CHAN*NO_IP_CHANNELS]; 
short inputsb[SAMPLES_PER_IP_CHAN*NO_IP_CHANNELS]; 





/* Function Prototypes */
HE_BYTE  Search_For_Modules(void);
void  Print_Device_Statistics(char *dev_str);
int   Query_Module_Type(HE_BYTE slot, HE_BYTE *reply, int buffer_size);
int   Initialise_Host_HSB(char *dev_name, HE_WORD board);
void  Print_Win32_Error(HE_DWORD errorcode);
void error(char *string, int Status);
void  Display_Help(void);
int Test_If_Proc_Module(char *dev_name, HE_WORD board, int slot);
void SendUserData(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data);
void SendUserDatanr(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data);
void Error(HE_DWORD errorcode);
void Check_Module(HE_BYTE slot);
int readback_user(HE_BYTE slot, HE_BYTE *reply, int buffer_size, HE_BYTE send_data);

/* Global Variables */
HE_HANDLE   hDevice			= NULL;
HE_IOSTATUS ReadIoStatus	= NULL;
HE_IOSTATUS WriteIoStatus	= NULL;
HE_HANDLE   hsbDevice		= NULL;

char command[70];

/************************************************************/
/*															*/
/*    GetWindowsVersion										*/
/*															*/
/************************************************************/

#define OSUNK	0
#define OSW31	1
#define OSW9x	2
#define OSNTx	3

void GetWindowsVersion(int *type, int *ver)
{
	OSVERSIONINFO osvi;

	ZeroMemory(&osvi, sizeof(OSVERSIONINFO));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (GetVersionEx((OSVERSIONINFO *)&osvi))
	{
		if (osvi.dwPlatformId==VER_PLATFORM_WIN32_NT)
		{
			*type = OSNTx;
			*ver  = osvi.dwMajorVersion;
		}
		else
		if (osvi.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS)
		{
			*type = OSW9x;
			*ver  = osvi.dwMinorVersion;
		}
		else
		{
			*type = OSW31;
			*ver  = osvi.dwMajorVersion;
		}
	}
	else
	{
		*type = OSUNK;
		*ver  = osvi.dwMajorVersion;
	}
}

/************************************************************/
/*															*/
/*    getregenv												*/
/*															*/
/* This function will first read a registry entry to find	*/
/* a HUNT environmental variable. If the variable is not	*/
/* found, getregenv will use 'getenv' to try to find the	*/
/* environmantal variable. Reason is that NT/W2K/XP don't	*/
/* seem to forward changes in environmental variables		*/
/* unless the PC is rebooted. This way we can read the		*/
/* latest change as done by confchk, but if a user only		*/
/* uses system environmental variables it'll also work.		*/
/*															*/
/************************************************************/

char *getregenv(const char *varname)
{
	static unsigned char kbuf[1024];
	DWORD    n=1024;
	int    type, ver;
	DWORD  ktype;

	GetWindowsVersion(&type, &ver);
	if (type==OSNTx)
	{
		char *envkey="SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment";
		HKEY  hkey;
		kbuf[0]=0;
		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, envkey, 0, KEY_QUERY_VALUE, &hkey)==ERROR_SUCCESS)
		{
			if (RegQueryValueEx(hkey, varname, NULL, &ktype, kbuf, &n)==ERROR_SUCCESS)
			{
				if ((ktype==REG_SZ)&&(n>0))
				{
					RegCloseKey(hkey);
					return (char *) kbuf;
				}
			}
			RegCloseKey(hkey);
		}
	}
	return getenv(varname);
}

/************************************************************/
/*															*/
/*    main													*/
/*															*/
/************************************************************/

main(int argc, char *argv[])
{
	char		*arg;
	char		 tmp[255]; 
	int			 n,i,ask;
	int			 fifo,fifo_in,fifo_out;
	char		*string; 
	char		*fifodev;
	int			 start, length;
	FILE		*fptr;
	short		 outputs[SAMPLES_PER_OP_CHAN*NO_OP_CHANNELS];
	short		 data1[SAMPLES_PER_IP_CHAN];
	short		 data2[SAMPLES_PER_IP_CHAN];
	int			*int_inputsa=(int*)inputsa;
	int			*int_inputsb=(int*)inputsb;
	int			 loop=0;

	/* Check the number of program arguments */;
	if (argc > 3) 
	{
		Display_Help();
		exit(-1);
	}

	/* Check arguments are valid */
	i = 1;
	verbose = 0;
	ask=0;
	while (i < argc) 
	{
		arg = argv[i++];
		if (arg[0] == '-') 
		{
			/* Program options */
			n = strlen(arg);
			if ((n == 2) && ((arg[1] == 'v') || (arg[1] == 'V'))) 
			{
				verbose = 1;
			}
			else if ((n == 2) && ((arg[1] == 'a') || (arg[1] == 'A'))) 
			{
				ask=1;
			}
			else 
			{
				Display_Help();
				exit(-1);
			}
		}
		else
		{
			Display_Help();
			exit(-1);
		}
	}

	string = NULL;
	if (ask==0)
	{
		/* Set up for board we're going to open */
		string=getregenv("HEAPIHSB");
		if (string==NULL)
		{	
			if (verbose)printf("No HEAPIHSB variable found, using HEAPIJTAG\n");
			string=getregenv("HEAPIJTAG");
			if (string==NULL)
			{
				if (verbose)printf("No HEAPIJTAG variable found\n");
			}
		}
	}
	
	if (string != NULL)
	{
		i = 0;
		while ((string[i]==' ') || (string[i]=='\t'))i++;
		start=i;
		while ((string[i]!=' ') && (string[i]!='\t'))i++;
		length=i-start; 
		strncpy(devname,&string[start] , length);
		i++;
		while ((string[i]==' ') || (string[i]=='\t'))i++;
		start=i;
		while ((string[i]!=' ') && (string[i]!='\t'))i++;
		length=i=start; 
		strncpy(tmp,&string[start] , length);
	    Board   = (HE_WORD)atoi(tmp);
	}
	else
	{
		printf("What board do you want to open (eg hep8a, hep9a, heb2a)? ");
		gets(devname);
		printf("Which board do you want to open? (eg 0,1,2...) ");
		gets(tmp);
		Board = atoi(tmp);
	}

	if( (Board<0) || (Board>15)) error("Bad Board Number",-1);
	if (verbose)printf("using device %s number %x \n",devname,Board);

	/* Open Board */
	Status = Initialise_Host_HSB(devname,Board);
	if(Status != HE_OK) error("Can't init HSB, exiting\n", Status);

	if (ask==1)
	{
		printf("Which slot is the FPGA module in? (1,2, 3 or 4) ");
		gets(tmp);
		Slot = atoi(tmp);
		if( (Slot<1)||(Slot>4)) error("Bad Slot Number",-1);
		if (verbose)printf("you entered slot %x\n",Slot);
	    Check_Module(Slot); 
	}
	else
	{
		/* nothing specified, search for possible modules and ask for shoices*/
		Slot=Search_For_Modules();
	}

	/* open the board and reset it*/
	Status = HeOpen1(devname, (HE_WORD)Board, "FifoA", &hDevice);
	if(Status != HE_OK) error("Open Failed, exiting\n", Status);
	/* reset the hardware*/
	HeReset(hDevice);

	/* Set up the FPGA for the demo */	
	// 0x40 = FIFO6
	// 0x20 = FIFO5
	// 0x10 = FIFO4
	// 0x08 = FIFO3
	// 0x04 = FIFO2
	// 0x02 = FIFO1
	// 0x01 = FIFO0
	if(stricmp(devname,"hep8a")==0)
	{
		if (Slot!=1)
		{
			printf("ERROR: For an HEPC8 the FPGA module must be in slot1\n");
			getchar();
			exit(0);
		}
		else
		{
			printf("Module is in slot1 and FIFO 1 will be used to connect to the Host\n");
			fifo=1;
			fifodev="FifoA";
		}
	}
	else
	if(stricmp(devname,"heb2a")==0)
	{
		switch (Slot)
		{
			case 2 : fifodev="FifoB"; fifo=1; break;
			default: fifodev="FifoA"; fifo=0; break;
		}
		if (fifodev!="FifoA")
		{
			Status = HeClose(&hDevice);
			if(Status != HE_OK) error("Failed close..., exiting\n", Status);
			Status = HeOpen1(devname, (HE_WORD)Board, fifodev, &hDevice);
			if(Status != HE_OK) error("Open Failed, exiting\n", Status);
		}
	}
	else
	{
		switch (Slot)
		{
			case 2 : fifodev="FifoB"; break;
			case 3 : fifodev="FifoC"; break;
			case 4 : fifodev="FifoD"; break;
			default: fifodev="FifoA"; break;
		}
		// The network file supplied with HeGraph connects 
		// always to fifo 0 on the module.
		fifo=0;
		if (fifodev!="FifoA")
		{
			Status = HeClose(&hDevice);
			if(Status != HE_OK) error("Failed close..., exiting\n", Status);
			Status = HeOpen1(devname, (HE_WORD)Board, fifodev, &hDevice);
			if(Status != HE_OK) error("Open Failed, exiting\n", Status);
		}
	}

	Status = HeInitIoStatus(hDevice, &ReadIoStatus); // Initialise read IoStatus
	if(Status != HE_OK) error("Can't init Read I/O, exiting\n", Status);
	Status = HeInitIoStatus(hDevice, &WriteIoStatus); // Initialise write IoStatus
	if(Status != HE_OK) error("Can't init Write I/O, exiting\n", Status);
	if(verbose)printf("Board Opened\n");

	/* send messages over the HSB to set the registers in the FPGA that select the HERON FIFO */
	/* that the FPGA will use to read and write */
	fifo_in=1<<fifo;
	SendUserData(devname, Board, Slot, 0, fifo_in);
	printf("Selected Input FIFO with control word %02x\n",fifo_in);

	fifo_out= 1<<fifo;
	SendUserData(devname, Board, Slot, 1, fifo_out);
	printf("Selected Output FIFO with control word %02x\n",fifo_out);


	/* if its an HEPC9 then set the HEART connections so that one host FIFO     */
	/* connects to each module slot. Then the host FIFO number determines which */
	/* module is being used */
	if(stricmp(devname,"hep9a")==0)
	{
		printf("Setting the HEART configuration on HEPC9\n");
		Status = HeClose(&hsbDevice);	
		string = getregenv("HEAPI_DIR");
		/* chech to see is a ntwork file exists in this directory*/
		fptr=fopen("network", "r");	
		if (fptr!=NULL)
		{
			fclose(fptr);
			sprintf(command,"%s\\heartconf\\heartconf network\n",string);
	    }
		else 
		{
			/* if not then look in the directory above in case we are running from a debig or release directory*/ 
			fptr=fopen("..\\network", "r");
			if (fptr!=NULL)
			{
				fclose(fptr);
				sprintf(command,"%s\\heartconf\\heartconf ..\\network\n",string);
			}
			else
			{
				printf("ERROR cannot locate the network file in this or the directory above\n");
				getchar();
				exit(0);
			}
		}
		/* call heart conf to set the network up */		
		printf(command);
		system(command);

		/* Open Board */
		Status = Initialise_Host_HSB(devname,Board);
		if(Status != HE_OK) error("Can't init HSB, exiting\n", Status);
	}


#ifdef DOINPUT
#if (NO_IP_CHANNELS==2)	
	SendUserDatanr(devname, Board, Slot, 2, 1); /* set 2 channels operation */
#else
	SendUserDatanr(devname, Board, Slot, 2, 0); /* set 1 channels operation */
#endif

#endif

#ifdef DOOUTPUT
#if (NO_OP_CHANNELS==2)	
	SendUserDatanr(devname, Board, Slot, 3, 1); /* set 2 channels operation */
	for(i=0;i<(SAMPLES_PER_OP_CHAN*NO_OP_CHANNELS);i+=2)
	{
		outputs[i]=(short)(0x1fff+ 0x1fff*( sin(i*(PI/512))));
		outputs[i+1]=(short)(0x1fff+ 0x1fff*( cos(i*(PI/512)))); 
	}		
#else
	SendUserDatanr(devname, Board, Slot, 3, 0); /* set 1 channels operation */
	for(i=0;i<(SAMPLES_PER_OP_CHAN*NO_OP_CHANNELS);i++)
	{
		outputs[i]=(short)(0x1fff+ 0x1fff*( sin(i*(PI/256))));
	}
#endif

#endif
	printf("Starting to loop\n");

#ifdef DOOUTPUT
	/* get an output buffer ahead*/
	
	Status = HeWrite(hDevice, outputs, (SAMPLES_PER_OP_CHAN*NO_OP_CHANNELS/2)*sizeof(HE_DWORD), WriteIoStatus);
	if(Status == HE_IoInProgress) HeWaitForIo(hDevice, WriteIoStatus);	
#endif
	while(1)
	{
#ifdef DOOUTPUT
		Status = HeWrite(hDevice, outputs, (SAMPLES_PER_OP_CHAN*NO_OP_CHANNELS/2)*sizeof(HE_DWORD), WriteIoStatus);
		
#endif
#ifdef DOINPUT
	     /* start to fill buffer a*/             
		Status = HeRead(hDevice, inputsa, (SAMPLES_PER_IP_CHAN*NO_IP_CHANNELS/2)*sizeof(HE_DWORD), ReadIoStatus);

		

		if (start==0)
		{		
			/* process the input data buffer b here if it is not the first time */

#if (NO_IP_CHANNELS==2)	
			/* split the two channels*/
			for (i=0;i<SAMPLES_PER_IP_CHAN;i++)
			{
				data1[i]=(short)(int_inputsa[i]&0xfff);
				data1[i]=data1[i]<<4;
				data1[i]=data1[i]>>4;
				data2[i]=(short)(int_inputsa[i]>>16&0xfff);
				data2[i]=data2[i]<<4;
				data2[i]=data2[i]>>4;
			}
#else
			for (i=0;i<SAMPLES_PER_IP_CHAN;i++)
			{
				data1[i]=(short)(inputsa[i]&0xfff);
				data1[i]=data1[i]<<4;
				data1[i]=data1[i]>>4;
			}
#endif
	
			/* do your processing */
		}
		else
		{
			start=0;
		}	
			
		/* wait for buffer a to be filled*/
		HeWaitForIo(hDevice, ReadIoStatus); 
#endif
#ifdef DOOUTPUT
		/* wait for output buffer to be sent*/
			
		HeWaitForIo(hDevice, WriteIoStatus); 

		/* start to output data*/
		Status = HeWrite(hDevice, outputs, (SAMPLES_PER_OP_CHAN*NO_OP_CHANNELS/2)*sizeof(HE_DWORD), WriteIoStatus);
	
#endif
#ifdef DOINPUT

	
		/* start to fill buffer b */
		Status = HeRead(hDevice, inputsb, (SAMPLES_PER_IP_CHAN*NO_IP_CHANNELS/2)*sizeof(HE_DWORD), ReadIoStatus);

		/* process the input data buffer a here */	

#if (NO_IP_CHANNELS==2)	
		/* split the two channels*/
		for (i=0;i<SAMPLES_PER_IP_CHAN;i++)
		{
			data1[i]=(short)(int_inputsa[i]&0xfff);
			data1[i]=data1[i]<<4;
			data1[i]=data1[i]>>4;
			data2[i]=(short)(int_inputsa[i]>>16&0xfff);
			data2[i]=data2[i]<<4;
			data2[i]=data2[i]>>4;
		}
#else
		for (i=0;i<SAMPLES_PER_IP_CHAN;i++)
		{
			data1[i]=(short)(inputsa[i]&0xfff);
			data1[i]=data1[i]<<4;
			data1[i]=data1[i]>>4;
		}
#endif
	
		/* do your processing */

		
		/* wait for buffer b to be filled*/
	
		HeWaitForIo(hDevice, ReadIoStatus); 

#endif	
#ifdef DOOUTPUT
		/* wait for output buffer to be sent*/
		HeWaitForIo(hDevice, WriteIoStatus); 	

#endif	
	} 

	/* Close down API and return */
	Status = HeClose(&hDevice);
	if(Status != HE_OK) error("Failed close..., exiting\n", Status);
	// Close down HSB device
	Status = HeClose(&hsbDevice);
	if(Status != HE_OK) error("Failed close..., exiting\n", Status);
}



void error(char *string, int Status)
{
	printf("Status %x\n", Status);
	printf("%s\n", string);
	Status = HeClose(&hDevice);
	getchar();
	exit(0);
}


void SendUserData(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data)
{
HE_BYTE user[2];
HE_BYTE rec_data[3];
int reply;

	// Send user data
	user[0]=(HE_BYTE) Address;
	user[1]=(HE_BYTE) Data;

	Status = HeHSBSendMessage(hsbDevice, USER_WRITE, slot, &user[0], 2, 1000);
	if(Status != HE_OK) error("Write module failed, exiting\n", Status);
// read it back to check
    reply =readback_user(slot,rec_data,3,Address);
    if(reply!=Data)printf("error hsb readback failed with %x\n",reply); 





}

void SendUserDatanr(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data)
{
HE_BYTE user[2];
HE_BYTE rec_data[3];
int reply;

	// Send user data
	user[0]=(HE_BYTE) Address;
	user[1]=(HE_BYTE) Data;

	Status = HeHSBSendMessage(hsbDevice, USER_WRITE, slot, &user[0], 2, 1000);
	if(Status != HE_OK) error("Write module failed, exiting\n", Status);




}


/********************************************************************************/
/*																				*/
/*	Initialise_Host_HSB															*/
/*																				*/
/*	This function sets up the host HSB interface of each board in the system.	*/
/*																				*/
/********************************************************************************/
int Initialise_Host_HSB(char *dev_name, HE_WORD board)
{



	/* Check parameters */
	if ((board < 0) || (board > 15)) {
		printf("Error: board ID is out of range (0 to 15)\n");
		return(-1);
	}

	Status = HeOpen(dev_name, board, HSB, &hsbDevice);
	if (Status != HE_OK) return(1);


	return(0);
}




/********************************************************************************/
/*																				*/
/*	Search_For_Modules															*/
/*																				*/
/*	This function searches all slots for either HERON-FPGA modules or HERON-IO  */
/*	that have an FPGA fitted.													*/
/*																				*/
/********************************************************************************/
HE_BYTE Search_For_Modules(void)
{
int i,n;
int found;
char device[30];
HE_BYTE reply[30];
int mod_type;
char tmp[30];

/* Search for all HERON-FPGA or I/O modules on Board */
found = 0;
printf("Searching for HERON-FPGA and HERON-IO modules\n");
for (Slot=1;Slot<5;Slot++) 
{
	mod_type = Test_If_Proc_Module(devname, Board, Slot);
	if (mod_type==-1) /*actually a non processor module with HSB*/
	{
		for (i=0;i<30;i++) reply[i] = 0;
		mod_type = Query_Module_Type(Slot,reply,30);
		if (mod_type < 0) exit(0);
		if ((mod_type == 2) || (mod_type == 4)) 
		{
			if (mod_type == 2) 
			{
				/* A HERON-FPGA module has been found */
				printf("\nA HERON-FPGA%d has been found in slot ",reply[1]);
				printf("%d of board %d\n",Slot,Board);
			}
			if (mod_type == 4) 
			{
				/* A HERON-IO with an FPGA has been found */
				printf("\nA HERON-IO%d has been found in slot ",reply[1]);
				printf("%d of board %d\n",Slot,Board);
			}
			device[0] = '\0';
			strcpy(device,(char *)&reply[3]);
			Print_Device_Statistics(device);
			printf("Is this the module you want to use the example with?\n");
			gets(tmp);
			n = strlen(tmp);
			if (n == 1) 
			{
				if ((tmp[0] == 'y') || (tmp[0] == 'Y')) 
				{
					found=1;
					break;
				}
			}
			
		}
	}
}

if (!found) 
	{
	printf("\nNo HERON-FPGA or HERON-IO modules were configured, exiting..\n");
	exit(0);
	}


return(Slot);
}

/********************************************************************************/
/*																				*/
/*	Check_Module																*/
/*																				*/
/*	This function searches all slots for either HERON-FPGA modules or HERON-IO  */
/*	that have an FPGA fitted.													*/
/*																				*/
/********************************************************************************/
void Check_Module(HE_BYTE slot)
{
int i;
int found;
char device[30];
HE_BYTE reply[30];
int mod_type;

/* check for HERON-FPGA or I/O modules in Slot */
found = 0;
printf("Checking for HERON-FPGA and HERON-IO modules\n");
mod_type = Test_If_Proc_Module(devname, Board, slot);
if (mod_type==0) /*actually no module fitted*/
	{
	printf("Error: there is no module in slot %d of board %d\n",slot,Board);
	exit(0);
	}
if (mod_type==1)  /* module with processor */
	{
	printf("Error: the module in slot %d of board %d is a processor module\n",slot,Board);
	exit(0);
	}
if (mod_type==2)  /* module with no HSB */		
	{
	printf("Error: the module in slot %d of board %d does not have a serial bus\n",slot,Board);
	exit(0);
	}
if (mod_type==-1) /*actually a non processor module with HSB*/
{
	for (i=0;i<30;i++) reply[i] = 0;
	mod_type = Query_Module_Type(slot,reply,30);
	if (mod_type < 0) exit(0);
	if ((mod_type == 2) || (mod_type == 4)) 
	{
		if (mod_type == 2) 
		{
			/* A HERON-FPGA module has been found */
			printf("\nA HERON-FPGA%d has been found in slot ",reply[1]);
			printf("%d of board %d\n",slot,Board);
		}
		if (mod_type == 4) 
		{
			/* A HERON-IO with an FPGA has been found */
			printf("\nA HERON-IO%d has been found in slot ",reply[1]);
			printf("%d of board %d\n",slot,Board);
		}
		device[0] = '\0';
		strcpy(device,(char *)&reply[3]);
		Print_Device_Statistics(device);
		found=1;
	}
}

if (!found) 
	{
	printf("\nNo HERON-FPGA or HERON-IO modules were configured, exiting..\n");
	exit(0);
	}

return;
}



/********************************************************************************/
/*																				*/
/*	Test_If_Proc_Module															*/
/*																				*/
/*	This function is used to find out what kind of module is present in each	*/
/*  slot.																		*/
/*																				*/
/********************************************************************************/
int Test_If_Proc_Module(char *dev_name, HE_WORD board, int slot)
{
	HE_DWORD info[21];
	HE_DWORD status;

	info[0]=HE_Get_ModuleTypes;
	status = HeGetBoardInfo(dev_name,board,info);
	if (status!=HE_OK) printf("error from get board info %x\n",status);
		
	
		if (info[(5*(slot-1))+1] ==0) return(0); /* no module*/
		if (info[(5*(slot-1))+2] ==1) return(1); /* processor module*/
		if (info[(5*(slot-1))+3] ==0) return(2); /* has no serial bus*/

	return(-1);
}

/********************************************************************************/
/*																				*/
/*	readback_user																*/
/*																				*/
/*	This function is used to read back from the FPGA							*/
/*  slot.																		*/
/*																				*/
/********************************************************************************/
int readback_user(HE_BYTE slot, HE_BYTE *reply, int buffer_size, HE_BYTE user_addr)
{
	HE_BYTE  message_type;
	HE_DWORD bytes_read;
	HE_BYTE  send_data[5];

	send_data[0]=user_addr;
	send_data[1]=1;



	Status = HeHSBSendMessage(hsbDevice, USER_READ, slot, send_data, 2, 1000);
	Status = HeHSBReceiveMessage(hsbDevice, &message_type, slot, reply, buffer_size, &bytes_read, 1000);
	if (Status == HE_OK) 
	{
		if (message_type != USER_READ_REPLY) 
		{
			printf("Error: expected user read reply\n");
			return(-1);
		}
		else 
		{

			return(reply[0]);
		}
	}

	return(Status);
}


/********************************************************************************/
/*																				*/
/*	Query_Module_Type															*/
/*																				*/
/*	This function is used to find out what kind of module is present in each	*/
/*  slot.																		*/
/*																				*/
/********************************************************************************/
int Query_Module_Type(HE_BYTE slot, HE_BYTE *reply, int buffer_size)
{
	HE_BYTE  message_type;
	HE_DWORD bytes_read;


	Status = HeHSBSendMessage(hsbDevice, MODULE_TYPE_QUERY, slot, NULL, 0, 1000);
	Status = HeHSBReceiveMessage(hsbDevice, &message_type, slot, reply, buffer_size, &bytes_read, 1000);
	if (Status == 0) {
		if (message_type != MODULE_TYPE_REPLY) {
			printf("Error: expected module type query reply\n");
			return(-1);
		}
		else return(reply[0]);
	}

	return(0);
}





/********************************************************************************/
/*																				*/
/*	Print_Device_Statistices													*/
/*																				*/
/*	This function displays information about each FPGA device that is found.	*/
/*																				*/
/********************************************************************************/
void Print_Device_Statistics(char *dev_str)
{
	int  i,a,b,c,d;
	char family[12];
	char size[8];
	char package[8];
	char speed[4];


	/* Derive the FPGA family from the device string */
	i = 0;
	a = 0;
	if (dev_str[i] == '2') family[a++] = dev_str[i++];
	family[a++] = dev_str[i++];
	family[a] = '\0';

	/* Derive the FPGA size from the device string */
	b = 0;
	while ((dev_str[i] >= '0') && (dev_str[i] <= '9')) {
		size[b++] = dev_str[i++];
	}
	size[b] = '\0';

	/* Derive the FPGA package from the device string */
	c = 0;
	if (dev_str[i] == 'e') {
		family[a++] = dev_str[i++];
		family[a] = '\0';
	}
	else if (dev_str[i] == 'x') {
		family[a++] = dev_str[i++];
		family[a++] = dev_str[i++];
		family[a] = '\0';
	}
	while (dev_str[i] != '-') {
		package[c++] = dev_str[i++];
	}
	i++;
	package[c] = '\0';

	/* Derive the FPGA package from the device string */
	d = 0;
	while (dev_str[i] != '\0') {
		speed[d++] = dev_str[i++];
	}
	speed[d] = '\0';

	if      (strcmp(family,"s")   == 0) strcpy(family,"Spartan\0");
	else if (strcmp(family,"2s")  == 0) strcpy(family,"Spartan-2\0");
	else if (strcmp(family,"sxl") == 0) strcpy(family,"Spartan-XL\0");
	else if (strcmp(family,"v")   == 0) strcpy(family,"Virtex\0");
	else if (strcmp(family,"2v")  == 0) strcpy(family,"Virtex-2\0");
	else if (strcmp(family,"ve")  == 0) strcpy(family,"Virtex-E\0");

	if (verbose) printf("Found device: \"%s\"\n",dev_str);
	printf("The FPGA on this module is a %s device with %sK gates",family,size);
	if (verbose) printf(", in a %s package\n",package);
	else printf("\n");
	printf("The FPGA speed grade is -%s\n",speed);

	return;
}

/********************************************************************************/
/*																				*/
/*	Error																		*/
/*																				*/
/*	General Error Routine														*/
/*																				*/
/********************************************************************************/
void Error(HE_DWORD errorcode)
{
	char text[120];


	printf("In error routine\n");
	HeErr2Text(Status, text);
	printf("Failed - %s, OsError = %ld", text, errorcode);
	Print_Win32_Error(errorcode);
	printf("\n");

	Status = HeClose(&hDevice);
	exit(1);
}
/********************************************************************************/
/*																				*/
/*	Display_Help																*/
/*																				*/
/*	This function displays help information about this program.					*/
/*																				*/
/********************************************************************************/
void Display_Help()
{


	printf("Prog_FIFO:Heron-FPGA and Heron-IO host side program for Getting Started example\n"); 
	printf("                     Version 1.0 15/2/01 HUNT ENGINEERING\n\n");
	printf("Usage:  Prog_FIFO [options] \n\n\n");
	printf("Options:\n\n");
	printf("-v          Verbose mode\n");
	printf("-a          Ask me - don't use environmental variables\n");
	

	return;
}

/********************************************************************************/
/*																				*/
/*	Print_Win32_Error															*/
/*																				*/
/*	Win32 Error Function														*/
/*																				*/
/********************************************************************************/
void Print_Win32_Error(HE_DWORD errorcode)
{
#ifdef _WIN32			/* This level of error code is only on WIN32 */
						/* don't even compile it if not WIN32!*/
	LPVOID lpMsgBuf;

	if(errorcode != 0) {
		printf(" (%s)\n", strerror(errorcode));

		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			errorcode, 
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), /* Default language*/
			(LPTSTR) &lpMsgBuf,
			0,
			NULL);
		printf("Sys Error - %s", lpMsgBuf);
	}
#endif				/* end of WIN32 specific error codes*/
}
