-- ==============================================================
-- FILE : HSB2.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of Example 2, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : HSB2
--  Date        : 07/02/2002
--  Author      : L. Bacquart - ALSE
--  Description : Using HUNT Message Interface
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     L. Bacquart    07-02-02    First Written
--  1.1     R. Williams    26-02-02    Changed names, added known
--                                     I/Os, modified IFD and OFD
--                                     implementation.
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--
-- ==============================================================


LIBRARY ieee;
  USE ieee.std_logic_1164.ALL;

entity HSB2 is
  port (
    RST           : in  std_logic;
    CLK           : in  std_logic;
    DIN           : in  std_logic_vector(7 downto 0);
    ADDR          : in  std_logic_vector(7 downto 0);
    AEN           : in  std_logic;
    WEN           : in  std_logic;
    REN           : in  std_logic;
    DONE          : in  std_logic;
    COUNT         : in  std_logic;
    CLEAR         : in  std_logic;
    READY         : out std_logic;
    SEND_MSG      : out std_logic;
    CE            : out std_logic;
    DOUT          : out std_logic_vector(7 downto 0);
    SEND_ID       : out std_logic;
    LAST_BYTE     : out std_logic;
    -- registers
    INFIFO_SEL    : out std_logic_vector(5 downto 0);
    OUTFIFO_SEL   : out std_logic_vector(5 downto 0);
    UPD_DONE      : in  std_logic;
    UPDATE        : out std_logic;
    SAMPLE        : out std_logic_vector(8 downto 0);
    OP_SEL        : out std_logic_vector(2 downto 0);
    DDS_FREQ      : out std_logic_vector(25 downto 0);
    DDS_EN        : out std_logic
  );
end HSB2;


architecture RTL of HSB2 is

  signal REG_0 : std_logic_vector(7 downto 0);
  signal REG_1 : std_logic_vector(7 downto 0);
  signal REG_2 : std_logic_vector(7 downto 0);
  signal REG_3 : std_logic_vector(7 downto 0);
  signal REG_4 : std_logic_vector(7 downto 0);
  signal REG_5 : std_logic_vector(7 downto 0);
  signal REG_6 : std_logic_vector(7 downto 0);
  signal REG_7 : std_logic_vector(7 downto 0);
  signal REG_8 : std_logic_vector(7 downto 0);

  signal EN    : std_logic_vector(8 downto 0);

begin

  SEND_MSG  <= '0';
  CE        <= '0';
  SEND_ID   <= '0';
  LAST_BYTE <= '0';

  INFIFO_SEL  <= REG_0(5 downto 0);
  OUTFIFO_SEL <= REG_1(5 downto 0);
  SAMPLE      <= REG_3(0) & REG_2;
  OP_SEL      <= REG_4(2 downto 0);
  DDS_FREQ    <= REG_8(1 downto 0) & REG_7 & REG_6 & REG_5;

  -- Decode address & output mux
  process(ADDR, REG_0, REG_1, REG_2, REG_3, REG_4)
  begin
    EN <= (others=>'0');
    case ADDR is
      when "00000000" => EN(0)<='1'; DOUT <= REG_0;
      when "00000001" => EN(1)<='1'; DOUT <= REG_1;
      when "00000010" => EN(2)<='1'; DOUT <= REG_2;
      when "00000011" => EN(3)<='1'; DOUT <= REG_3;
      when "00000100" => EN(4)<='1'; DOUT <= REG_4;
      when "00000101" => EN(5)<='1'; DOUT <= REG_5;
      when "00000110" => EN(6)<='1'; DOUT <= REG_6;
      when "00000111" => EN(7)<='1'; DOUT <= REG_7;
      when "00001000" => EN(8)<='1'; DOUT <= REG_8;
      when others     => DOUT <= (others=>'-');
    end case;
  end process;

  -- DATA_REGs
  process (CLK,RST)
  begin
   if RST = '1' then
         REG_0 <= (others=>'0');
         REG_1 <= (others=>'0');
         REG_2 <= (others=>'0');
         REG_3 <= (others=>'0');
         REG_4 <= ("00000100"); --filter2 output selected
         -- Initialise to 10MHz 
         REG_5 <= ("01100110");
         REG_6 <= ("01100110");
         REG_7 <= ("01100110");
         REG_8 <= ("00000000");
   elsif  rising_edge(CLK) then
     if WEN='1' then
        if EN(0)='1' then
          REG_0 <= DIN;
        end if;
        if EN(1)='1' then
          REG_1 <= DIN;
        end if;
        if EN(2)='1' then
          REG_2 <= DIN;
        end if;
        if EN(3)='1' then
          REG_3 <= DIN;
        end if;
        if EN(4)='1' then
          REG_4 <= DIN;
        end if;
        if EN(5)='1' then
          REG_5 <= DIN;
        end if;
        if EN(6)='1' then
          REG_6 <= DIN;
        end if;
        if EN(7)='1' then
          REG_7 <= DIN;
        end if;
        if EN(8)='1' then
          REG_8 <= DIN;
        end if;
     end if;
   end if;
  end process;

 -- DDS Enable
  process (RST, CLK)
  begin
    if RST='1' then
     DDS_EN <= '0';
    elsif rising_edge(CLK) then
      DDS_EN <= WEN AND EN(8);
    end if;
  end process;


  -- READY
  process (RST, CLK)
  begin
    if RST='1' then
      READY <= '0';
    elsif rising_edge(CLK) then
      READY <= WEN OR REN;
    end if;
  end process;

  -- UPDATE samples
  process(RST, CLK)
  begin
    if RST='1' then
      UPDATE <= '0';
    elsif rising_edge(CLK) then
      if EN(3)='1' AND WEN='1' then
        UPDATE <= '1';
      elsif UPD_DONE='1' then
        UPDATE <= '0';
      end if;
    end if;
  end process;

end RTL;

