-- ==============================================================
-- FILE : UP_CONV.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of DDC Example, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : UP_CONV
--  Date        : 07/02/2002
--  Author      : L. Bacquart - ALSE
--  Description : DAC data flow Control
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0       T.Hollis       20/03/03   first written
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity UP_CONV is
  port (
    RST             : in  std_logic;
    SCLK            : in  std_logic;
    DATA_A          : in  std_logic_vector(15 downto 0);
    DATA_B          : in  std_logic_vector(15 downto 0);
    DATA_AB_RDY     : in  std_logic;
    DAC_A           : out std_logic_vector(13 downto 0)
  );
end UP_CONV;


architecture RTL of UP_CONV is

    component ddc_cic2
    port (
      DIN  :  in std_logic_VECTOR(13 downto 0);
      ND   :  in std_logic;
      RFD  :  out std_logic;
      CLK  :  in std_logic;
      DOUT :  out std_logic_VECTOR(42 downto 0);
      RDY  :  out std_logic
     );
   end component;

   component ddc_add
   port (
     A     : in   std_logic_VECTOR(13 downto 0);
     B     : in   std_logic_VECTOR(13 downto 0);
     Q     : out  std_logic_VECTOR(13 downto 0);
     CLK   : in   std_logic
    );
   end component;

   component ddc_mult
    port (
    clk          : in  std_logic;
    a            : in  std_logic_VECTOR(11 downto 0);
    b            : in  std_logic_VECTOR(11 downto 0);
    q            : out std_logic_VECTOR(15 downto 0)
    );
   end component;

   component dds2
    port (
    CLK          : in  std_logic;
    SINE         : out std_logic_VECTOR(11 downto 0)
    );
   end component;



  signal ADD_OP     : std_logic_vector(13 downto 0);
  signal ADD_OP_RDY : std_logic;

  signal CIC2_OP    : std_logic_vector(42 downto 0);
  signal CIC2_OP_RDY : std_logic;
  signal CIC2_RFD : std_logic;

  signal SINE       : std_logic_vector(11 downto 0);
  signal MULT_A_IP  : std_logic_vector(11 downto 0); 
  signal MULT_OP    : std_logic_vector(15 downto 0);
 


begin

iDDC2 : ddc_cic2
    port map (
      DIN  =>  ADD_OP,
      ND   =>  ADD_OP_RDY,
      RFD  =>  CIC2_RFD,
      CLK  =>  SCLK,
      DOUT =>  CIC2_OP,
      RDY  =>  CIC2_OP_RDY);

      MULT_A_IP(11) <= CIC2_OP(42);
      MULT_A_IP(10 downto 0) <= CIC2_OP(38 downto 28);
--(39 downto 29)


iADD : ddc_add
   port map (
     A     => DATA_A(13 downto 0),
     B     => DATA_B(13 downto 0),
     Q     => ADD_OP,
     CLK   => SCLK);
--LATENCY OF 1 SO REGISTER THE RDY
  process(RST, SCLK)
  begin
    if RST='1' then
      ADD_OP_RDY <= '0';
    elsif rising_edge(SCLK) then    
      ADD_OP_RDY <= DATA_AB_RDY;
    end if;
  end process;

  iddc_mult : ddc_mult
        port map (
            clk  => SCLK,
            a    => MULT_A_IP,
            b    => SINE,
            q    => MULT_OP);

  DAC_A(13) <= (NOT MULT_OP(15));
  DAC_A(12 downto 0) <= MULT_OP(13 downto 1);


iDDS2 : dds2
        port map (
            CLK  => SCLK,
            SINE => SINE);


end RTL;
