-- ==============================================================
-- FILE : CIC.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of DDC Example ,for the HERON-IO2 module.
-- It contains the CIC fiters
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : CIC
--  Date        : 16/01/2003
--  Author      : T. HOLLIS
--  Description : Digital Down Converter
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     T. Hollis     16/01/2003    First Written
--
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity CIC is
  port (
    RST           : in  std_logic;
    SCLK          : in  std_logic;
    CIC_IP_I      : in  std_logic_vector(13 downto 0);
    CIC_IP_Q      : in  std_logic_vector(13 downto 0);
    CIC_OP_I      : out std_logic_vector(13 downto 0);
    CIC_OP_RDY_I  : out std_logic;
    CIC_OP_Q      : out std_logic_vector(13 downto 0);
    CIC_OP_RDY_Q  : out std_logic
  );
end CIC;


architecture RTL of CIC is

  component ddc_cic1
      port (
      DIN          : in std_logic_VECTOR(13 downto 0);
      ND           : in std_logic;
      RFD          : out std_logic;
      CLK          : in std_logic;
      DOUT         : out std_logic_VECTOR(32 downto 0);
      RDY          : out std_logic);
  end component;

  signal ND        : std_logic;
  signal RFD_I     : std_logic;
  signal RFD_Q     : std_logic;
  signal DOUT_I    : std_logic_VECTOR(32 downto 0);
  signal DOUT_Q    : std_logic_VECTOR(32 downto 0);


begin

  -- CIC starts after reset
  -- ==========
  process(RST, SCLK)
  begin
    if RST='1' then
      ND  <= '0';
    elsif rising_edge(SCLK) then
      ND <= RFD_I AND RFD_Q;
    end if;
  end process;


icic1_I : ddc_cic1
        port map (
            DIN => CIC_IP_I,
            ND => ND,
            RFD => RFD_I,
            CLK => SCLK,
            DOUT => DOUT_I,
            RDY => CIC_OP_RDY_I);

-- TRUNCATE TO 14 BIT OP
CIC_OP_I(13) <= DOUT_I(32);
CIC_OP_I(12 downto 0) <= DOUT_I(31 downto 19);

icic1_Q : ddc_cic1
        port map (
            DIN => CIC_IP_Q,
            ND => ND,
            RFD => RFD_Q,
            CLK => SCLK,
            DOUT => DOUT_Q,
            RDY => CIC_OP_RDY_Q);

-- TRUNCATE TO 14 BIT OP
CIC_OP_Q(13) <= DOUT_Q(32);
CIC_OP_Q(12 downto 0) <= DOUT_Q(31 downto 19);

end RTL;
