-- ==============================================================
-- FILE : MIXER.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of DDC Example ,for the HERON-IO2 module.
-- It contains the DDS and the phase(I) and quadrature(Q) multipliers
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : MIXER
--  Date        : 16/01/2003
--  Author      : T. HOLLIS
--  Description : Digital Down Converter
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     T. Hollis     16/01/2003    First Written
--  1.1     T. Hollis     08/07/03      Multiplier changed
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity MIXER is
  port (
    RST           : in  std_logic;
    SCLK          : in  std_logic;
    ADC_A         : in  std_logic_vector(11 downto 0);
    MULT_OP_I     : out std_logic_vector(13 downto 0);
    MULT_OP_Q     : out std_logic_vector(13 downto 0);
    SIN_OP        : out std_logic_vector(11 downto 0);
    DDS_FREQ      : in std_logic_vector(25 downto 0);
    DDS_EN        : in std_logic
  );
end MIXER;


architecture RTL of MIXER is

component dds
    port (
    DATA         : in  std_logic_VECTOR(25 downto 0);
    WE           : in  std_logic;
    CLK          : in  std_logic;
    SINE         : out std_logic_VECTOR(11 downto 0);
    COSINE       : out std_logic_VECTOR(11 downto 0) );
end component;

component ddc_mult
    port (
    clk          : in  std_logic;
    a            : in  std_logic_VECTOR(11 downto 0);
    b            : in  std_logic_VECTOR(11 downto 0);
    q            : out std_logic_VECTOR(15 downto 0));
end component;


  --signal DDS_FREQ  : std_logic_VECTOR(25 downto 0);
  signal SIN       : std_logic_VECTOR(11 downto 0);
  signal COS       : std_logic_VECTOR(11 downto 0);

  signal DOUT_I    : std_logic_VECTOR(15 downto 0); 
  signal DOUT_Q    : std_logic_VECTOR(15 downto 0);

  signal UDP_R     : std_logic;
  signal UDP_RR    : std_logic;
  signal DDS_WE    : std_logic;

  signal xSCLK     : std_logic;

begin
SIN_OP <= SIN;

xSCLK <= SCLK;

  -- DDS write enable after reset
  -- ==========
  process(RST, SCLK)
  begin
    if RST='1' then
      UDP_R  <= '0';
      UDP_RR <= '0';
    elsif rising_edge(SCLK) then
      UDP_R <= ( (NOT RST) AND (NOT UDP_R) AND (NOT UDP_RR) ) OR DDS_EN ;
      UDP_RR <= UDP_R OR UDP_RR;
    end if;
  end process;

  DDS_WE <= UDP_R;


  idds : dds
      port map (
         DATA   => DDS_FREQ,
         WE     => DDS_WE,
         CLK    => xSCLK,
         SINE   => SIN,
         COSINE => COS);

  iddc_mult_I : ddc_mult
        port map (
            clk  => xSCLK,
            a    => ADC_A,
            b    => COS,
            q    => DOUT_I);

   -- Truncate output to 14 bits
   MULT_OP_I(13) <= DOUT_I(15);
   MULT_OP_I(12 downto 0) <= DOUT_I(13 downto 1);

  iddc_mult_Q : ddc_mult
        port map (
            clk  => xSCLK,
            a    => ADC_A,
            b    => SIN,
            q    => DOUT_Q );

   -- Truncate output to 14 bits
   MULT_OP_Q(13) <= DOUT_Q(15);
   MULT_OP_Q(12 downto 0) <= DOUT_Q(13 downto 1);
 
end RTL;
