-- ==============================================================
-- FILE : IP_MEM.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of FFT_Example, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : IP_MEM
--  Date        : 13/05/03
--  Author      : T. Hollis 
--  Description : input memory 
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     T. Hollis      11-06-03    First Written
--                                     
--                                     
--                                     
--                                     
--                                     
--                                    
--                                    
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity IP_MEM is
  port (
    RST             : in  std_logic;
    FCLK            : in  std_logic;
    SCLK            : in  std_logic;
    INFIFO_SEL      : in  std_logic_vector(5 downto 0);
    INFIFO_DVALID   : in  std_logic_vector(5 downto 0);
    INFIFO_READ_REQ : out std_logic_vector(5 downto 0);
    DATA_IN         : in  std_logic_vector(31 downto 0);
    RD_MEM_ADDR     : in  std_logic_vector(9 downto 0);
    MEM_DOUT        : out std_logic_vector(31 downto 0)
  );
end IP_MEM;


architecture RTL of IP_MEM is

   component ram1024x32
	port (
       addra       : in  std_logic_VECTOR(9 downto 0);
       addrb       : in  std_logic_VECTOR(9 downto 0);
       clka        : in  std_logic;
       clkb        : in  std_logic;
       dina        : in  std_logic_VECTOR(31 downto 0);
       doutb       : out std_logic_VECTOR(31 downto 0);
       wea         : in  std_logic);
   end component;

  signal READ_REQ   : std_logic;
  signal DVALID     : std_logic;
  signal WRITE      : std_logic;
  signal WR_A       : std_logic;
  signal WR_ADDR_A  : std_logic_vector(9 downto 0);
  signal WRAcounter : unsigned(9 downto 0);

begin

  READ_REQ <= '1';

  INFIFO_READ_REQ(0) <= READ_REQ AND INFIFO_SEL(0);
  INFIFO_READ_REQ(1) <= READ_REQ AND INFIFO_SEL(1);
  INFIFO_READ_REQ(2) <= READ_REQ AND INFIFO_SEL(2);
  INFIFO_READ_REQ(3) <= READ_REQ AND INFIFO_SEL(3);
  INFIFO_READ_REQ(4) <= READ_REQ AND INFIFO_SEL(4);
  INFIFO_READ_REQ(5) <= READ_REQ AND INFIFO_SEL(5);

  DVALID <= (INFIFO_DVALID(0) AND INFIFO_SEL(0)) OR
            (INFIFO_DVALID(1) AND INFIFO_SEL(1)) OR
            (INFIFO_DVALID(2) AND INFIFO_SEL(2)) OR
            (INFIFO_DVALID(3) AND INFIFO_SEL(3)) OR
            (INFIFO_DVALID(4) AND INFIFO_SEL(4)) OR
            (INFIFO_DVALID(5) AND INFIFO_SEL(5));

  WR_A <= DVALID;

  -- WRITE address
  process(RST, FCLK)
  begin
    if RST='1' then
      WRAcounter <= (others=>'0');
    elsif rising_edge(FCLK) then
      if WR_A='1' then
        WRAcounter <= WRAcounter + 1;
      end if;
    end if;
  end process;

  WR_ADDR_A <= std_logic_vector(WRAcounter);

  iRAM1024x32 : ram1024x32
     port map (
       addra   => WR_ADDR_A,
       addrb   => RD_MEM_ADDR,
       clka    => FCLK,
       clkb    => SCLK,
       dina    => DATA_IN,
       doutb   => MEM_DOUT,
       wea     => WR_A );

end RTL;
