-- ==============================================================
-- FILE : SIG_SEL.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of FFT_Example, for the HERON-IO2 module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : SIG_SEL
--  Date        : 23/04/03
--  Author      :T. Hollis
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     T. Hollis      23-04-03    First Written

--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.numeric_std.all;

entity SIG_SEL is
  port (
    RST           : in  std_logic;
    SCLK          : in  std_logic;
    ADC_A         : in  std_logic_vector(11 downto 0);
    MEM_DOUT      : in  std_logic_vector(31 downto 0);
    FFT_IP_I      : out std_logic_vector(15 downto 0);
    FFT_IP_Q      : out std_logic_vector(15 downto 0);
    IP_SEL        : in  std_logic_vector(1 downto 0);
    DDS_FREQ      : in  std_logic_vector(25 downto 0);
    DDS_EN        : in  std_logic
  );
end SIG_SEL;


architecture RTL of SIG_SEL is

  component dds
     port (
      DATA         : in  std_logic_VECTOR(25 downto 0);
      WE           : in  std_logic;
      CLK          : in  std_logic;
      SINE         : out std_logic_VECTOR(15 downto 0);
      COSINE       : out std_logic_VECTOR(15 downto 0));
  end component;


  signal UDP_R        : std_logic;
  signal UDP_RR       : std_logic;
  signal DDS_WE       : std_logic;

  signal SINE         : std_logic_vector(15 downto 0);
  signal COSINE       : std_logic_vector(15 downto 0);

begin

  -- IP MULTIPLEXER(PHASE)
  process (IP_SEL, COSINE, ADC_A)
    begin
       case IP_SEL is
            when "00" =>
        FFT_IP_I <= ADC_A & "0000";
            when "01" =>
        FFT_IP_I <= COSINE;
            when "10" =>
        FFT_IP_I <= MEM_DOUT(15 downto 0);
            when others =>
        FFT_IP_I <= ADC_A & "0000";
       end case;
  end process;

  -- IP MULTIPLEXER(QUAD)
  process (IP_SEL, SINE)
    begin
       case IP_SEL is
            when "00" =>
        FFT_IP_Q <= (others=> '0');
            when "01" =>
        FFT_IP_Q <= SINE;
           when "10" =>
        FFT_IP_Q <= MEM_DOUT(31 downto 16);
            when others =>
        FFT_IP_Q <= (others=> '0');
       end case;
  end process;

   -- DDS write enable after reset
  -- ==========
  process(RST, SCLK)
  begin
    if RST='1' then
      UDP_R  <= '0';
      UDP_RR <= '0';
    elsif rising_edge(SCLK) then
      UDP_R <= ( (NOT RST) AND (NOT UDP_R) AND (NOT UDP_RR) ) OR DDS_EN ;
      UDP_RR <= UDP_R OR UDP_RR;
    end if;
  end process;

  DDS_WE <= UDP_R;

  iDDS : dds
   port map (
    DATA        => DDS_FREQ,
    WE          => DDS_WE,
    CLK         => SCLK,
    SINE        => SINE,
    COSINE      => COSINE );

end RTL;
