/********************************************************************************/
/*																				*/
/* fft_example.c                                	            for heron-io2v2 */
/*    																			*/
/* Example for a HERON-IO2V2 module used with a DSP module.            	        */
/* It works with the fft example bistream loaded into a HERON-IO2V2  			*/
/*																				*/
/*	   2003 Hunt Engineering													*/
/*	    All Rights Reserved														*/
/*																				*/
/* 	Ver	Date		Author	Comments											*/
/*	---	----		------	--------											*/
/*																				*/
/*	1.0	26/6/01	PW		Created.											    */ 
/*	2.0	04/3/03	TH		Modified for DDC example.							    */ 
/*	3.0	11/5/03	TH		Modified for FFT example.							    */
/********************************************************************************/


#include <stdio.h>
#include <std.h>
#include <swi.h>
#include <tsk.h>  
#include <math.h>
#include "heron4.h"

#pragma FUNC_EXT_CALLED(maintask)


/************************************/
/* FIFO information for Heron_API   */
/************************************/
#define FIFONO	2		/* FIFO through which module communicates with FPGA */
HeronFIFO *rfifo;		/* Handle for read FIFO  */
HeronFIFO *wfifo;   	/* Handle for write FIFO */        
HeronHSB *hsb;          /* handle for HSB accesses */

extern void HEL_UnpackSe16bit2chan (short *input, unsigned short bitmask, short *output, int sam_per_chan, short *errors);

/************************************/
/* Arrays for data in FPGA test     */
/************************************/
#define ARRAY_SIZE 1024	/*number of words to write to FPGA*/
#define DATA_MASK 0xffff  
#define SAMPLES_PER_CHAN ARRAY_SIZE
                                 
#define FPGA_TO_DSP 3
#define FPGA_SLOT	2  
#define USER_WRITE			8
#define USER_READ			9      
#define USER_READ_REPLY		10

#define LUT_ADDR_BITS  10        /* Number of bits for DDS LUT */ 
#define TOT_ACC_BITS  26        /* Number of bits for DDS LUT */
#define SAMPLE_FREQ    100      /* Sample frequency in MHz */ 

#define AM_INDEX 0.5
#define FM_INDEX 1.0
  
#define PI 3.1415926535         /* So we can calculate angles in radians */  
  
short *shortdata;		
int *readdata;			/* results read back from FPGA */	
int *chadata;	
int *chbdata;
int *ampldata;	

int i,slot,board;
  
/* allocate the errors array */
#pragma DATA_ALIGN(errors,4);   
short errors[2];     
  
   
/********************************************************************************/
/*																				*/
/*	readback_user																*/
/*																				*/
/*	This function is used to read back from the FPGA							*/
/*																				*/
/********************************************************************************/
int readback_user(unsigned short targetslot, unsigned char *reply, int buffer_size, int user_addr)
{
	int message_type;
	unsigned char send_data[5];   
	unsigned int done;
	int status;

	send_data[0]=user_addr;
	send_data[1]=1;


   	status = HeronHsbSendMessage(hsb,USER_READ,board,targetslot,send_data,2);  
   	status = HeronHsbReceiveMessage(hsb,&message_type,board,targetslot,reply,buffer_size,&done);
	if (status == HERON_OK) 
	{
		if (message_type != USER_READ_REPLY) 
		{
			printf("SLOT%d:Error: expected user read reply   \n",slot);
			return(-1);
		}
		else 
		{       
			if (done!=1)
			{
				printf("SLOT%d:Error: expected one byte, recieved %d  \n",slot,done);
				return(-1);
			}
			else
			{	
			return(reply[0]);
			}
		}
	}

	return(status);
}    



/****************************************************/
/* makes an HSB message to send to the FPGA module	*/
/* it is a "user" type message to write to the      */
/* application FPGA. In the example1 bitstream address */
/* 0 sets the FIFO to use for input to the FPGA, and */
/* address 1 sets the FIFO to use for writes from the */
/* FPGA												*/
/************************************/


void SendUserData(unsigned short targetslot, int Address, int Data)
{
unsigned char user[2];
unsigned char rec_data[3];
int reply;
int status;

	/* Send user data */
	user[0]=Address;
	user[1]=Data;
        
	status = HeronHsbSendMessage(hsb,USER_WRITE,board,targetslot,user,2);        
        
//    printf("sent\n");    
	if(status != HERON_OK) 
		{
		printf("SLOT%d:HSB Write module failed with %x, exiting\n",targetslot, status);  
		exit(0);
		}
/* read it back to check*/        
    reply =readback_user(targetslot,rec_data,2,Address);
    if(reply!=Data)
    {
    printf("SLOT%d:error hsb readback failed with %x\n",targetslot,reply); 
    exit(0);
    }

}         
      
/****************************************************/
/* makes an HSB message to send to the FPGA module	*/
/* it is a "user" type message to write to the      */
/* application FPGA. In the example1 bitstream address */
/* 0 sets the FIFO to use for input to the FPGA, and */
/* address 1 sets the FIFO to use for writes from the */
/* FPGA												*/
/************************************/


void SendUserDatanr(unsigned short targetslot, int Address, int Data)
{
unsigned char user[2];
int status;

	/* Send user data */
	user[0]=Address;
	user[1]=Data;
        
	status = HeronHsbSendMessage(hsb,USER_WRITE,board,targetslot,user,2);        
        
//    printf("sent\n");    
	if(status != HERON_OK) 
		{
		printf("SLOT%d:HSB Write module failed with %x, exiting\n",targetslot, status);  
		exit(0);
		}

}          
   
int main(void)
{     
}


/************************************/
/* Maintask - main processing task	*/
/* Performs the test by reading the data 	*/
/************************************/

int maintask(void)
{
/* WARNING the default stack size for this task ( maintask) defined in    */
/* the cdb file is 8192 bytes                                             */
/* This means that you cannot statically declare more than 2048 integers  */
/* in this task.                                                          */
/* You can increase the stack size allowed by changing the settings for   */
/* the task in the cdb file.                                              */
/* If you need more than 2048 integers, it is better to use Malloc/Free   */
/* to allocate your data buffers 					  */



int status,fifo;          
int i,j,coshalf,sinhalf;
double treal,timag,sum_sqs;
double trealsq,timagsq;
int sig_source,window,fwd_tran,comb;
float freq; /* required frequency in MHz */
float del_theta;
int int_del_theta;
FILE *wp; 
FILE *rp;
char readfile[32];
char *wfile = "fft_op3t.dat";
char *rfile = "fft_op2t.dat";
int r;

                                    
readdata = (int *) malloc(ARRAY_SIZE * sizeof(int));
shortdata = (short *) malloc(ARRAY_SIZE * sizeof(int));
ampldata = (int *) malloc((ARRAY_SIZE) * sizeof(int));
     

chadata = (int *) malloc(ARRAY_SIZE * sizeof(int));
chbdata = (int *) malloc(ARRAY_SIZE * sizeof(int));

if ((readdata==0) || (ampldata==0) ||(shortdata==0) || (chadata==0) || (chbdata==0))
	{
	printf("Memory allocation error, exiting...\n");
	exit(0);
	}   

	
slot = HeronModId() && 0xf;
board= (HeronModId() && 0xf0)>>4;
            
/* open the HSB device */
hsb = HeronHsbOpen();
if (hsb == NULL) {
	printf("SLOT%d:Error: Call to HeronHsbOpen failed, error = %d\n",slot,heronerr); 
	exit(0);
}          
          
/* open the FIFO for reading*/
    rfifo=HeronOpenFifo(FIFONO,"r");
    if (rfifo==NULL)
     {
     printf("Cannot open fifo %d for read. Error %d\n",FIFONO,heronerr);
     exit(-1); 	       
     }

/* open the FIFO for writing*/
    wfifo=HeronOpenFifo(FIFONO,"w");
    if (wfifo==NULL)
     {
     printf("Cannot open fifo %d for write. Error %d\n",FIFONO,heronerr);
     exit(-1); 	
     } 

     
	fifo=1<<FPGA_TO_DSP;  
	SendUserData(FPGA_SLOT, 0, fifo); 
	SendUserData(FPGA_SLOT, 1, fifo); 

      
	


/* Start I/O System */
	HeronConfigOff();
	
/* Commented out area of the program that would write a file        */
/* containing a phase and quadrature signal suitable for loading    */
/* into the FPGA memory as a time domain signal with a sine wave.   */
//   for (i=0;i<1024;i++)
//     { 
//      j = (i%20);
//      coshalf=0x0000ffff & (int)( 0x7fff*( cos(j*(PI/10)) ));
//      sinhalf=0x0000ffff & (int)( 0x7fff*( sin(j*(PI/10)) ));       
//      chadata[i]=(coshalf | ( sinhalf<<16) );   
//     } 
/* Open file for writing */     
//     wp = fopen(wfile,"wb");                                 
//     if (wp == NULL)
//       {
//	    printf("Sorry, cannot open %s for writing.\n", wfile);
//	    return 0;
//       }  		 
//     r = fwrite(chadata,4,1024,wp); 
//     if (r !=1024)
//       {
//	   printf("Only wrote %d to %s .\n", r,wfile);
//	   return 0;
//       }     
//   fclose(wp);
 
	 
	/*Enter the signal source required for the FFT*/ 
	sig_source = -1; 
    do
    {
    printf("Signal Source:-  0 --> sig gen. \n");
    printf("                 1 --> dds. \n"); 
    printf("                 2 --> mem. \n");    
    printf("Signal Source = ");
	scanf("%d",&sig_source);
	if(sig_source==0) printf("\nSignal Source is Signal Generator\n"); 
	if(sig_source==1) printf("\nSignal Source is DDS\n");
	if(sig_source==2) printf("\nSignal Source is MEMORY\n");		
	printf("\n");
	} while((sig_source != 0) && (sig_source != 1) && (sig_source != 2) );
	 
	
	/*If the sinal source is Memory */
    /*Read a data file to be loaded into FPGA memory*/
	if(sig_source==2)
	{
	 
     printf("Read file = ");
	 scanf("%s",&readfile);	
	 /*printf("read file is %s \n",readfile); */
     rfile = readfile; 
	 
	  /* Open file for reading */   
      rp = fopen(rfile,"rb");                                 
      if (rp == NULL)
       {
	    printf("Sorry, cannot open %s for reading.\n", rfile);
	    return 0;
       }
	  printf("Reading data file %s .\n", rfile);       
      r = fread(chadata,4,1024,rp);
      if (r !=1024)
       {
	    printf("Only read %d from %s .\n", r,rfile);
	    return 0;
       }              
      fclose(rp);
        
     /* Send data to memory in FPGA */      
      status = HeronWrite(wfifo,chadata,1024);
      if (status == HERON_IO_IN_PROGRESS)
	   {
		status = HeronWaitIo(wfifo);
	   }
	  if (status != HERON_OK)
	   {
        printf("Cannot write fifo %d. Error %d \n",FIFONO,heronerr);
		exit(0);
       }
      
      /*If source is Memory Forward or Inverse */
      /*Transform required.                    */
      fwd_tran = -1;      
      do
       {
        /*Forward or reverse transform*/
        printf("Forward/Inverse transform:-  0 --> Inverse. \n");
        printf("                             1 --> Forward. \n");
        printf("Forward/Inverse = "); 
	    scanf("%d",&fwd_tran);
	    if(fwd_tran==0) printf("\nInverse Fourier Transform\n"); 
	    if(fwd_tran==1) printf("\nForward Fourier Transform\n");		
	    printf("\n");    	
	   } while((fwd_tran != 0) && (fwd_tran != 1));
	    
    } /* end if(sig_source==2) */
    else
    fwd_tran=1; /*Forward Transform */
    
    /* If Forward Transform what window is required*/
    if( fwd_tran==1)
    {
     window = -1;	
     do
     {
     printf("Window:-         0 --> Keiser-Bessel. \n");
     printf("                 1 --> Rectangular. \n");
     printf("Window = ");
	 scanf("%d",&window);
	 if(window==0) printf("\nWindow is Keiser-Bessel\n"); 
	 if(window==1) printf("\nWindow is Rectangular\n");	
	 printf("\n");
	 } while((window != 0) && (window != 1));
	}
	else
	window = 1; /*Rectangular Window for inverse transform */	
	
	/* Send FFT configuration register value*/
	comb = ((sig_source | (window<<2) | (fwd_tran<<3))); 
	/* printf("comb = %x \n",comb); */    	
    SendUserData(FPGA_SLOT, 4, comb );
    	
	/*If the signal source is the DDS what frequency is required */
	if(sig_source == 1)
	{
    printf("Enter the required DDS frequency in MHz \n"); 
    printf("The frequency resolution of the DDS is 1.5Hz \n");
    printf("Frequency = "); 
	scanf("%f",&freq);
	printf("\n");
	
    /* calculate the code for the required DDS frequency */	
	 del_theta = ( (freq/SAMPLE_FREQ)*pow(2,LUT_ADDR_BITS)) * pow(2,(TOT_ACC_BITS-LUT_ADDR_BITS));
    /* printf("del_theta = %f ....",del_theta); */
     int_del_theta = (int)del_theta;
    /* printf("int_del_theta = %d : %x \n",int_del_theta,int_del_theta); */
     
    /* Send the required frequency to the DDS */     
    SendUserDatanr(FPGA_SLOT, 5 ,( int_del_theta&0xff) ); 
	SendUserDatanr(FPGA_SLOT, 6 ,((int_del_theta>>8)&0xff)); 	
 	SendUserDatanr(FPGA_SLOT, 7 ,((int_del_theta>>16)&0xff)); 
	SendUserDatanr(FPGA_SLOT, 8 ,((int_del_theta>>24)&0x03));
	/*writing to address 8 will initiate new DDS frequency */
	}
	
	 
	/* grab a file of data */	
    /* Request a block of data */
//	printf("sending request\n");
	SendUserDatanr(FPGA_SLOT, 2 ,((ARRAY_SIZE)&0xff)); /* bottom part of count */
	SendUserDatanr(FPGA_SLOT, 3 ,(((ARRAY_SIZE)>>8)&0xff)); /* top part of count and request to start*/	
 
//	printf("request sent -- now startng the read\n");  
  	status = HeronRead(rfifo,readdata,ARRAY_SIZE);
	if (status != HERON_IO_IN_PROGRESS)
	{      
		printf("error cannot start the read\n");
  		exit(-1);
  	}                  

	status = HeronWaitIo(rfifo);
   	if (status != HERON_OK) 
	{
		printf("Cannot read fifo %d. Error %d\n",FIFONO,heronerr);
		exit(-1);
	} 
	
/*The commented out area can be used to save a data file*/
/*containing the FFT output in a format suitable for    */
/*loading into signal source memory.                    */	
/* Open file for writing */     
//     wp = fopen(wfile,"wb");                                 
//     if (wp == NULL)
//       {
//	    printf("Sorry, cannot open %s for writing.\n", wfile);
//	    return 0;
//       }  		 
//     r = fwrite(readdata,4,1024,wp); 
//     if (r !=1024)
//       {
//	   printf("Only wrote %d to %s .\n", r,wfile);
//	   return 0;
//       }     
//   fclose(wp);	
			

/* The FPGA is now set up, so start grabbing blocks of FFT output data. */             
while (1)
{ 
 	 
 
 /* Request a block of data */
//	printf("sending request\n");
	SendUserDatanr(FPGA_SLOT, 2 ,((ARRAY_SIZE)&0xff)); /* bottom part of count */
	SendUserDatanr(FPGA_SLOT, 3 ,(((ARRAY_SIZE)>>8)&0xff)); /* top part of count and request to start*/	
 
//	printf("request sent -- now startng the read\n");  
  	status = HeronRead(rfifo,readdata,ARRAY_SIZE);
	if (status != HERON_IO_IN_PROGRESS)
	{      
		printf("error cannot start the read\n");
  		exit(-1);
  	}                  

	status = HeronWaitIo(rfifo);
   	if (status != HERON_OK) 
	{
		printf("Cannot read fifo %d. Error %d\n",FIFONO,heronerr);
		exit(-1);
	} 
/*	printf("Reading Test Pattern complete\n");*/	
	

	
	HEL_UnpackSe16bit2chan ((short *)readdata, DATA_MASK, shortdata ,SAMPLES_PER_CHAN, errors);
					
     for (i=0;i<(ARRAY_SIZE);i++)
     {
     /* For Inverse transform gain of 64 */ 
     if( fwd_tran==0)
       { 
        chadata[i] = 64 * shortdata[i];            /* Phase data */
        chbdata[i] = 64 * shortdata[i+ARRAY_SIZE]; /* Quadrature data */
        ampldata[i] = 0;
       } 
     else
     /* For Forward Transform */
       {
        chadata[i] = 0;
        chbdata[i] = 0;  
        /* Calculate the magnitude of the FFT output */
        treal = (double)shortdata[i];
        timag = (double)shortdata[i+ARRAY_SIZE];
        trealsq = treal*treal;
        timagsq = timag*timag;     
        sum_sqs = (trealsq + timagsq);
        if(sum_sqs <= 0.0) sum_sqs = 1.0;         
        ampldata[i] = (int)(10.0 * log10(sum_sqs) );
      
        /* Tweek Factors */
        if(window==0) /* Kaiser-Bessel */
        ampldata[i] = ampldata[i]-76; 
        if(window==1) /* Rectangular */
        ampldata[i] = ampldata[i]-84;
      } 
      
     }              
	         	
	
		                        
	HeronDigioOut(0xf);
}
  
  
  
             
            
status = HeronClose(rfifo);
status = HeronClose(wfifo);    
status = HeronHsbClose(hsb);            
             
             
return(0);
}
