
#if _LINUX
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <math.h>
#else
#include "stdafx.h"
#include "resource.h"
#include "commdlg.h"
#include "stdio.h"
#include "math.h"
#endif


#define	PI		3.141592653589793		/* definition of pi */

typedef struct Complex
{
	float imag;
	float real;
} COMPLEX;

void spdftr(float *x, COMPLEX *y, int N)
{
	int   m, k;
	float arg;

	for (m=0; m<N/2; m++)
	{
		y[m].real = x[0];
		y[m].imag = 0.0;
		for (k=1; k<N; k++)
		{
			arg = (float)2.0*(float)PI*(float)m*(float)k/(float)N;
			y[m].real += x[k]*cos(arg);
			y[m].imag -= x[k]*sin(arg);
		}
	}
}

void dft(int N, COMPLEX *X)
{
	int		n, k;
    float	arg;
	float	Xr[1024];
	float	Xi[1024];
	float	Wr, Wi;
    
	for (k=0; k<N; k++)
	{
		Xr[k] = 0.0;
		Xi[k] = 0.0;
		for (n=0; n<N; n++)
		{
			arg   = ((float)2.0*(float)PI*(float)k*(float)n)/(float)N;
			Wr    = (/*(float)32767.0 * */cos(arg));
			Wi    = (/*(float)32767.0 * */sin(arg)); 
			Xr[k] = Xr[k] + X[n].real * Wr + X[n].imag * Wi;
			Xi[k] = Xi[k] + X[n].imag * Wr - X[n].real * Wi;
		}
	 }

/*
	 for (k=0;k<N;k++)
	 {
		X[k].real = (short)(Xr[k]>>15);
		X[k].imag = (short)(Xi[k]>>15);
	 }
*/
	 for (k=0; k<N; k++)
	 {
		 X[k].real = Xr[k]/(float)N;
		 X[k].imag = Xi[k]/(float)N;
	 }
}

void ftwiddle(float w[], int N)
{
	float arg;
	int   i;
	
	arg   = (float)2.0*(float)PI/(float)N;
	for (i=0; i<N; i+=2)
	{
		w[i] = (float)cos(((float)i)*arg);
	}
	for (i=1; i<N; i+=2)
	{
		w[i] = (float)sin(((float)i)*arg);
	}
}

void cfftr2_dit_c(float* x, float* w, short n)
{
   short n2, ie, ia, i, j, k, m;
   float rtemp, itemp, c, s;

   n2 = n;
   ie = 1;

   for (k=n; k > 1; k >>= 1)
   {
      n2 >>= 1;
      ia = 0;
      for(j=0; j < ie; j++)
      {
         c = w[2*j];
         s = w[2*j+1];
         for(i=0; i < n2; i++)
         {
            m         = ia + n2;
            rtemp     = c * x[2*m]   + s * x[2*m+1];
            itemp     = c * x[2*m+1] - s * x[2*m];
            x[2*m]    = x[2*ia]   - rtemp;
            x[2*m+1]  = x[2*ia+1] - itemp;
            x[2*ia]   = x[2*ia]   + rtemp;
            x[2*ia+1] = x[2*ia+1] + itemp;
            ia++;
         }
         ia += n2;
      }
      ie <<= 1;
   }
}

int ReverseTable(unsigned short *idigits, unsigned short *jdigits, int n, int m)
{
	int k, l, i, bit, index;

	for (l=0, k=0; k<n; k++)
	{
		index = 0;
		for (i=0; i<m; i++)
		{
			bit    = (k&(1<<i))>>i;
			index |= (bit<<(m-i-1));
		}
		if (index > k)
		{
			idigits[l] = k;
			jdigits[l] = index;
			l++;
		}
	}

	return l;
}

void digit_reverse(double *yx, unsigned short *JIndex, unsigned short *IIndex, int count)
{
	int				i;
	unsigned short	I, J;
	double 			YXI, YXJ;

	for (i = 0; i<count; i++)
	{
		I		= IIndex[i];
		J		= JIndex[i];
		YXI		= yx[I];
		YXJ		= yx[J];
		yx[J]	= YXI;
		yx[I]	= YXJ;
	}
}

float				tw[1024];
unsigned short		IIndex[10];
unsigned short		JIndex[10];

void fft(int N, COMPLEX *X)
{
	int m, count;

	switch (N)
	{
	case 1024 : m=10; break;
	case  512 : m= 9; break;
	case  256 : m= 8; break;
	case  128 : m= 7; break;
	case   64 : m= 6; break;
	case   32 : m= 5; break;
	case   16 : m= 4; break;
	case    8 : m= 3; break;
	case    4 : m= 2; break;
	case    2 : m= 1; break;
	default   : return;
	}
	ftwiddle(tw, N/2);
	count = ReverseTable(IIndex, JIndex, N/2, m-1);

	cfftr2_dit_c ((float  *)X, tw, N);
	digit_reverse((double *)X, JIndex, IIndex, count);
}
